from espace import *
import entities
from random import uniform
from game import Game

import effects

# The BlasterProjectile, StormProjectile, RailBeam and Rocket classes in this
# file are entities, maintained by the "entities" namespace.
#
# think() is called regularily, with the difference in time since the last time
# it was called as a paramter.
#
# hit() is called whenever an entity intersects with another entity.
#
# addToScene() is called each frame, to let the entity add itself to
# the render list.
#
# The specialMatrix and specialOrigin members are used for "special attack
# mode", for instance to make the rocket camera work.

### Blaster ###

class BlasterProjectile:
  def __init__(self, origin, direction, owner):
    target = origin + direction * Blaster.speed * 2

    trace = Trace()
    Game.currentMap.rayTrace(origin, target, trace, 1)

    self.origin = origin
    self.direction = direction
    self.end = Vector3(trace.end)
    self.time = trace.fraction * 2
    self.radius = 10
    self.spawntime = 0
    self.owner = owner

  def think(self, deltaTime):
    # Update the age of the projectile
    self.time -= deltaTime

    # Move the projectile Blaster.speed units per second
    self.origin = self.origin + self.direction * deltaTime * Blaster.speed

    # A map wall was hit
    if self.time < 0:
      # Add sparks for visual effect
      effects.addSparks(self.end, -self.direction)

      # Play hit sound
      source = Blaster.soundSources[Blaster.soundSource]
      source.stop()
      source.setPosition(self.end)
      source.setGain(1.0)
      source.play(Blaster.hit)

      Blaster.soundSource += 1
      if Blaster.soundSource == len(Blaster.soundSources):
        Blaster.soundSource = 0

      # Remove this projectile from the list
      entities.remove(self)

  def hit(self, player):
    if player == self.owner:
      return

    if player.damage(20):
      self.owner.kills.append(player)
      Game.message = "%s was killed by %s's blaster" % (player.name, self.owner.name)
      Game.messageTime = 3

    entities.remove(self)

  def addToScene(self):
    entity = RefEntity()

    entity.type = RefEntity.Beam
    entity.renderfx = 0
    entity.origin = self.origin
    entity.oldOrigin = self.origin - self.direction * 200
    entity.customShader = Blaster.shader
    entity.frame = 5

    Renderer.appendRefEntity(entity)
    Renderer.addLight(self.origin, 200, Color(127, 127, 255))

class Blaster:
  name = "Blaster"
  specialMatrix = 0
  specialOrigin = 0

  # Handle for shader (texture)
  shader = 0

  # Sources (channels) for playing impact sounds
  soundSources = list()
  soundSource = 0

  # Sound handles
  attack = None
  hit = None

  # The minimum amount of time between each call to addProjectile
  minDelta = 0.5

  # The amount of units per second the projectile moves
  speed = 4000

  def init():
    for a in range(0, 8):
      source = SoundSource()
      source.setReferenceDistance(100)
      Blaster.soundSources.append(source)

    Blaster.shader = Shader.acquireHandle("textures/weapons/blaster", False)
    Blaster.attack = Sound.acquire("sounds/blaster.wav")
    Blaster.hit = Sound.acquire("sounds/blaster_hit.wav")

  def addProjectile(player):
    # The origin of the projectiles relative to the ship
    p0 = player.rd.origin - player.up * 10 - player.right * 20
    p1 = player.rd.origin - player.up * 10 + player.right * 20

    # Add the new projectiles to the list
    entities.append(BlasterProjectile(p0, player.direction, player))
    entities.append(BlasterProjectile(p1, player.direction, player))

    source = player.soundSources[player.soundSource]
    source.stop()
    source.setGain(0.1)
    source.play(Blaster.attack)

    player.soundSource += 1
    if player.soundSource == len(player.soundSources):
      player.soundSource = 0

  init = staticmethod(init)
  addProjectile = staticmethod(addProjectile)

### Rocket launcher ###

class Rocket:
  def __init__(self, origin, direction, right, up, owner):
    target = origin + direction * RocketLauncher.speed * 5

    trace = Trace()
    Game.currentMap.rayTrace(origin, target, trace, 1)

    self.origin = origin
    self.direction = direction
    self.end = Vector3(trace.end)
    self.time = trace.fraction * 5
    self.lastSmoke = self.time
    self.lastFire = self.time
    self.right = right
    self.up = up
    self.radius = 10
    self.spawntime = 0
    self.owner = owner

  def explode(self):
    # Add explosion effect
    effects.addExplosion(self.end - self.direction * 100)

    for p in Game.localPlayers:
      distance = (p.rd.origin - self.end).magnitude()

      if distance > 500:
        continue

      if p.damage((500 - distance) / 8):
        if p == self.owner:
          p.suicides += 1
          Game.message = "%s was killed by his own rocket" % (p.name)
        else:
          self.owner.kills.append(p)
          Game.message = "%s was killed by %s's rocket" % (p.name, self.owner.name)
        Game.messageTime = 3

    # Remove this projectile from the list
    entities.remove(self)

    if self == RocketLauncher.last:
      RocketLauncher.last = 0
      RocketLauncher.specialMatrix = 0

  def think(self, deltaTime):
    # Update the age of the projectile
    self.time -= deltaTime

    # Move the projectile Blaster.speed units per second
    self.origin = self.origin + self.direction * deltaTime * RocketLauncher.speed

    # A map wall was hit
    if self.time < 0:
      self.explode()

      return

    if self.lastSmoke - self.time > 0.02:
      effects.addSmoke(self.origin - self.direction * 78)
      self.lastSmoke -= 0.05

    if self.lastFire - self.time > 0.001:
      effects.addFire(self.origin - self.direction * 78)
      self.lastFire -= 0.005

    if self == RocketLauncher.last:
      RocketLauncher.specialMatrix = Matrix3x3(self.direction, self.right, self.up)
      RocketLauncher.specialOrigin = self.origin + self.direction * -70 + self.right * 5 + self.up * uniform(10, 10.2)

  def hit(self, player):
    if player == self.owner:
      return

    self.end = self.origin
    self.explode()

  def addToScene(self):
    entity = RefEntity()

    entity.type = RefEntity.Model
    entity.renderfx = 0
    entity.modelHandle = RocketLauncher.rocket
    entity.axis = Matrix3x3(-self.direction, -self.right, self.up).inverse()
    entity.origin = self.origin
    entity.frame = 0
    entity.skinNum = 0
    entity.customSkin = 0
    entity.customShader = RocketLauncher.rocketShader
    entity.color = Color(255, 255, 255)

    Renderer.appendRefEntity(entity)

    Renderer.addLight(self.origin, 300, Color(255, 127, 63))
  
class RocketLauncher:
  name = "Rocket launcher"
  specialMatrix = 0
  specialOrigin = 0

  last = 0

  rocket = 0
  rocketShader = 0

  speed = 2000

  minDelta = 1.0

  def init():
    RocketLauncher.rocket = Model.acquire("models/weapons/rocket.md3")
    RocketLauncher.rocketShader = Shader.acquireHandle("simple:models/weapons/rocket.png")

  def addProjectile(player):
    if not player.rockets:
      return

    player.rockets -= 1

    if player.leftRocket:
      origin = player.rd.origin - player.up * 20 - player.right * 20
    else:
      origin = player.rd.origin - player.up * 20 + player.right * 20

    player.leftRocket = not player.leftRocket

    RocketLauncher.last = Rocket(origin, player.direction, player.right, player.up, player)

    entities.append(RocketLauncher.last)

  def rotate(pitch, yaw):
    if not RocketLauncher.last:
      return

    last = RocketLauncher.last

    RocketLauncher.specialMatrix.rotate(last.right, -pitch * 2)
    RocketLauncher.specialMatrix.rotate(last.up, yaw * 2)

    last.direction = Vector3(RocketLauncher.specialMatrix(0, 0),
                             RocketLauncher.specialMatrix(1, 0),
                             RocketLauncher.specialMatrix(2, 0))
    last.right = Vector3(RocketLauncher.specialMatrix(0, 1),
                         RocketLauncher.specialMatrix(1, 1),
                         RocketLauncher.specialMatrix(2, 1))
    last.up = Vector3(RocketLauncher.specialMatrix(0, 2),
                      RocketLauncher.specialMatrix(1, 2),
                      RocketLauncher.specialMatrix(2, 2))

    trace = Trace()

    target = last.origin + last.direction * RocketLauncher.speed * 5

    Game.currentMap.rayTrace(last.origin, target, trace, 1)

    last.end = Vector3(trace.end)
    last.time = trace.fraction * 5

  init = staticmethod(init)
  addProjectile = staticmethod(addProjectile)
  rotate = staticmethod(rotate)

### Storm ###

class StormProjectile:
  def __init__(self, origin, direction, light, owner):
    target = origin + direction * Storm.speed * 2

    trace = Trace()
    
    Game.currentMap.rayTrace(origin, target, trace, 1)

    self.origin = origin
    self.direction = direction
    self.end = Vector3(trace.end)
    self.time = trace.fraction * 2
    self.light = light
    self.radius = 10
    self.spawntime = 0
    self.owner = owner

  def think(self, deltaTime):
    # Update the age of the projectile
    self.time -= deltaTime

    # Move the projectile 3000 units per second
    self.origin = self.origin + self.direction * deltaTime * Storm.speed

    # A map wall was hit
    if self.time < 0:
      entities.remove(self)

  def hit(self, player):
    if player == self.owner:
      return

    if player.damage(10):
      self.owner.kills.append(player)
      Game.message = "%s was killed by %s's Storm" % (player.name, self.owner.name)
      Game.messageTime = 3

    entities.remove(self)

  def addToScene(self):
    entity = RefEntity()

    entity.type = RefEntity.Sprite
    entity.renderfx = 0
    entity.origin = self.origin
    entity.customShader = Storm.shader
    entity.radius = 5

    Renderer.appendRefEntity(entity)

    if self.light:
      Renderer.addLight(self.origin, 100, Color(255, 255, 127))

class Storm:
  name = "Storm"
  specialMatrix = 0
  specialOrigin = 0

  # The minimum amount of time between each call to addProjectile
  minDelta = 0.1

  # Handle for shader (texture)
  shader = 0

  # Sound handles
  attack = None

  # The amount of units per second the projectile moves
  speed = 3000

  def init():
    Storm.shader = Shader.acquireHandle("textures/effects/spark", False)
    Storm.attack = Sound.acquire("sounds/storm.wav")

  def addProjectile(player):
    if not player.energy:
      return

    player.energy -= 1

    origin = player.rd.origin - player.up * 40

    for a in range(1, 10):
      dir = player.direction * uniform(0.9, 1.1) + player.right * uniform(-0.2, 0.2) + player.up * uniform(-0.2, 0.2)

      entities.append(StormProjectile(origin, dir, a == 1, player))

    source = player.soundSources[player.soundSource]
    source.stop()
    source.setGain(0.1)
    source.play(Storm.attack)

    player.soundSource += 1
    if player.soundSource == len(player.soundSources):
      player.soundSource = 0

  init = staticmethod(init)
  addProjectile = staticmethod(addProjectile)

### Rail Gun ###

class RailBeam:
  def __init__(self, origin, direction, angle, owner):
    target = origin + direction * 10000

    trace = Trace()

    Game.currentMap.rayTrace(origin, target, trace, 1)

    self.origin = origin
    self.angle = angle
    self.end = Vector3(trace.end)
    self.time = 1
    self.radius = 10
    self.spawntime = 0
    self.owner = owner

    for p in Game.localPlayers:
      if p == owner:
        continue

      if Collision.intersect(origin, self.end, p.rd.origin, p.radius):
        if p.damage(100):
          owner.kills.append(p)
          Game.message = "%s was killed by %s's rail gun" % (p.name, self.owner.name)
          Game.messageTime = 3

  def think(self, deltaTime):
    self.time -= deltaTime

    if self.time < 0:
      entities.remove(self)

  def hit(self, player):
    pass

  def addToScene(self):
    entity = RefEntity()

    entity.type = RefEntity.RailRings
    entity.customShader = RailGun.shader
    entity.renderfx = 0
    entity.origin = self.origin
    entity.oldOrigin = self.end
    entity.color = Color(self.time * 0.25, self.time * 0.5, self.time)
    entity.time = self.angle

    Renderer.appendRefEntity(entity)

    entity = RefEntity()

    entity.type = RefEntity.RailCore
    entity.customShader = RailGun.shader
    entity.renderfx = 0
    entity.origin = self.origin
    entity.oldOrigin = self.end
    entity.color = Color(self.time * 0.8, self.time * 0.8, self.time * 0.8)
    entity.time = self.angle

    Renderer.appendRefEntity(entity)

class RailGun:
  name = "Rail gun"
  specialMatrix = 0
  specialOrigin = 0
  minDelta = 1.5

  # Handle for shader (texture)
  shader = 0

  # Sound handles
  attack = None

  def init():
    RailGun.attack = Sound.acquire("sounds/railgun.wav")
    RailGun.shader = Shader.acquireHandle("textures/weapons/railgun", False)

  def addProjectile(player):
    if not player.slugs:
      return

    player.slugs -= 1

    origin = player.rd.origin - player.up * 20

    entities.append(RailBeam(origin, player.direction,
                             uniform(0, 10), player))

    source = player.soundSources[player.soundSource]
    source.stop()
    source.setGain(1.0)
    source.play(RailGun.attack)

    player.soundSource += 1
    if player.soundSource == len(player.soundSources):
      player.soundSource = 0

  init = staticmethod(init)
  addProjectile = staticmethod(addProjectile)

