/***************************************************************************
                             file.cc  -  Buffered file I/O
                               -------------------
      copyright            : (C) 2003 by Morten Hustveit
      email                : morten@debian.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <algorithm>
#include <map>

#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <fcntl.h>
#include <errno.h>
#include <unistd.h>
#include <string.h>
#ifdef _POSIX_MAPPED_FILES
#  include <sys/mman.h>
#endif
#ifdef WIN32
#  include <io.h>
#endif

#include <espace/file.h>
#include <espace/string.h>
#include <espace/output.h>
#include <espace/plugins.h>

File::EntryList File::entries;

static bool operator==(const FileEntry& entry, const String& string)
{
  return entry.name == string;
}

bool File::readDirectory(const char* _directory, const char* prefix)
{
  String directory = _directory;

  if(directory.beginsWith("./"))
  {
    char pwd[PATH_MAX];

    getcwd(pwd, PATH_MAX);

    directory = String(pwd) + directory.right(directory.length() - 1);
  }
  else if(directory.beginsWith("~/"))
  {
    directory = String(getenv("HOME")) + directory.right(directory.length() - 1);
  }

  uint oldSize = entries.size();

  if(!*prefix)
    esInfo << "Scanning directory \"" << directory << "\"... ";

  DIR* dir = opendir(directory);

  if(!dir)
  {
    if(!*prefix)
      esInfo << strerror(errno) << "." << std::endl;

    return false;
  }

  struct dirent* ent;

#ifndef WIN32
  struct stat buf;
  stat(directory, &buf);

  int subdirsLeft = buf.st_nlink - 2; // "directory" and "directory/."
#else
  int subdirsLeft = -1;
#endif

  entries.reserve(8192); // More than the amount of files in RtCW with expansion packs.

  while(0 != (ent = readdir(dir)))
  {
    if(!strcmp(ent->d_name, ".") || !strcmp(ent->d_name, ".."))
      continue;

    String fileName = String(directory) + "/" + ent->d_name;

    if(subdirsLeft)
    {
      if(readDirectory(fileName, String(prefix) + ent->d_name + "/"))
      {
        --subdirsLeft;

        continue;
      }
    }

    File file(fileName, 4);

    if(!file.isOpen())
      continue;

    PluginMap(Archive)::iterator i;

    for(i = Plugin::archive.begin(); i != Plugin::archive.end(); ++i)
    {
      if(i->second->canHandle(file))
      {
        i->second->scan(file, fileName, entries);

        break;
      }
    }

    // If there's no compatible archive plugins, treat as ordinary file
    if(i == Plugin::archive.end())
    {
      FileEntry entry;

      entry.type = 0;
      entry.name = (String(prefix) + String(ent->d_name).replace('\\', '/')).toLower();
      entry.archive = fileName;

      entries.push_back(entry);
    }
  }

  closedir(dir);

  if(!*prefix)
    esInfo << "done." << std::endl;

  return true;
}

String File::findSingle(const char* name)
{
  EntryList::const_iterator i = std::find(entries.begin(), entries.end(),
                                          String(name).toLower());

  if(i == entries.end())
    return String::null;

  return i->name;
}

String File::realName(const char* name)
{
  EntryList::const_iterator i = std::find(entries.begin(), entries.end(),
                                          String(name).toLower());

  if(i == entries.end())
    return String::null;

  return i->archive;
}

File::File(const char* fileName, uint bufferSize)
  : bufferSize(bufferSize),
    bufferPosition(0),
    position(0),
    writing(false),
    archive(0)
{
#ifndef WIN32
  if(fileName[0] != '/' && !entries.empty())
#else
  if((!fileName[0] || fileName[1] != ':') && !entries.empty())
#endif
  {
    String searchName = String(fileName).toLower();

    EntryList::const_iterator i;

    for(i = entries.begin(); i != entries.end(); ++i)
      if((*i).name == searchName)
        break;

    if(i == entries.end())
    {
      fd = -1;

      return;
    }

    if((*i).type) // File is within an archive
    {
      archive = Plugin::archive[(*i).type];

      fd = archive->open(*i);

      if(fd < 0)
        return;

      fileSize = (*i).size;

      if(fileSize < this->bufferSize)
        this->bufferSize = fileSize;

      if(!this->bufferSize)
        return;

      buffer = new unsigned char[this->bufferSize];

      archive->read(fd, buffer, this->bufferSize);

      return;
    }
    else // File is not in an archive
    {
      fd = open((*i).archive, O_RDONLY);

      if(fd < 0)
        return;
    }
  }
  else // fileName[0] == '/'
  {
    fd = open(fileName, O_RDONLY);

    if(fd < 0)
      return;
  }

  struct stat buf;

  fstat(fd, &buf);

  fileSize = buf.st_size;

#ifdef _POSIX_MAPPED_FILES
  this->bufferSize = fileSize;

  if(!this->bufferSize)
    return;

  buffer = reinterpret_cast<unsigned char*>(mmap(0, this->bufferSize, PROT_READ, MAP_PRIVATE, fd, 0));

  if(buffer == MAP_FAILED)
  {
    esWarning << "Failed to mmap \"" << fileName << "\": " << strerror(errno) << "." << std::endl;

    close(fd);

    fd = -1;

    return;
  }
#else // !_POSIX_MAPPED_FILES
  if(fileSize < this->bufferSize)
    this->bufferSize = fileSize;

  if(!this->bufferSize)
    return;

  buffer = new unsigned char[this->bufferSize];

  ::read(fd, buffer, this->bufferSize);
#endif
}

File::File(const char* fileName, bool truncate, uint bufferSize = 65536)
  : bufferSize(bufferSize),
    bufferPosition(0),
    writing(true),
    archive(0)
{
  fd = open(fileName, O_WRONLY | O_CREAT | (truncate ? O_TRUNC : O_APPEND), 0644);

  if(fd < 0)
    return;

  if(bufferSize)
    buffer = new unsigned char[bufferSize];
}

File::~File()
{
  if(fd >= 0)
  {
    if(writing && bufferPosition)
      flush();

    if(archive)
    {
      archive->close(fd);
    }
    else
    {
      close(fd);
    }

    if(bufferSize)
    {
      if(archive || writing)
      {
        delete [] buffer;
      }
      else
      {
#ifdef _POSIX_MAPPED_FILES
        munmap(buffer, bufferSize);
#else // !_POSIX_MAPPED_FILES
        delete [] buffer;
#endif
      }
    }
  }
}

uint File::modified()
{
  struct stat fileState;

  fstat(fd, &fileState);

  return fileState.st_mtime;
}

uint8_t File::getU8()
{
#ifdef _POSIX_MAPPED_FILES
  if(archive && position + 1 > bufferSize)
#else // !_POSIX_MAPPED_FILES
  if(position + 1 > bufferSize)
#endif
  {
    if(position > bufferSize)
      sys_seek(bufferPosition);

    bufferPosition += position;
    position = 0;

    if(bufferPosition + bufferSize < fileSize)
      sys_read(buffer, bufferSize);
    else
      sys_read(buffer, fileSize - bufferPosition);
  }

  return buffer[position++];
}

uint16_t File::getU16()
{
#ifdef _POSIX_MAPPED_FILES
  if(archive && position + 2 > bufferSize)
#else // !_POSIX_MAPPED_FILES
  if(position + 2 > bufferSize)
#endif
  {
    uint16_t ret;

    ret = getU8();
    ret |= getU8() << 8;

    return ret;
  }

  position += 2;

  return
      buffer[position - 2]
    | (buffer[position - 1] << 8);
}

uint32_t File::getU32()
{
#ifdef _POSIX_MAPPED_FILES
  if(archive && position + 4 > bufferSize)
#else // !_POSIX_MAPPED_FILES
  if(position + 4 > bufferSize)
#endif
  {
    uint32_t ret;

    ret = getU16();
    ret |= getU16() << 16;

    return ret;
  }

  position += 4;

  return
      buffer[position - 4]
    | (buffer[position - 3] << 8)
    | (buffer[position - 2] << 16)
    | (buffer[position - 1] << 24);
}

String File::readLine()
{
  if(eof())
    return String::null;

  String ret("");

  do
  {
    char c = (position < bufferSize) ? buffer[position++] : getU8();

    if(c == '\r')
      continue;

    if(c == '\n')
      break;

    ret += c;
  }
  while(bufferPosition + position < fileSize);

  return ret;
}

void File::read(void* buffer, uint count)
{
#ifdef _POSIX_MAPPED_FILES
  if(!archive || position + count < bufferSize)
#else // !_POSIX_MAPPED_FILES
  if(position + count < bufferSize)
#endif
  {
    memcpy(buffer, this->buffer + position, count);

    position += count;
  }
  else
  {
    int amount = 0;

    if(position < bufferSize)
    {
      amount = bufferSize - position;

      memcpy(buffer, this->buffer + position, amount);
    }
    else if(position > bufferSize)
    {
      sys_seek(bufferPosition + position);
    }

    sys_read(static_cast<char*>(buffer) + amount, count - amount);

    position += count;
  }
}

uint8_t* File::data()
{
#ifdef _POSIX_MAPPED_FILES
  if(archive)
#endif
  {
    if(bufferSize == fileSize)
      return buffer;

    uint8_t* newBuffer = new uint8_t[fileSize];
    memcpy(newBuffer + bufferPosition, buffer, bufferSize);

    if(bufferPosition + bufferSize < fileSize)
    {
      sys_seek(bufferPosition + bufferSize);

      sys_read(newBuffer + bufferPosition + bufferSize, fileSize - (bufferPosition + bufferSize));
    }

    if(bufferPosition > 0)
    {
      sys_seek(0);

      sys_read(newBuffer, bufferPosition);
    }

    delete [] buffer;

    position += bufferPosition;
    bufferPosition = 0;
    bufferSize = fileSize;
    buffer = newBuffer;
  }

  return buffer;
}

void File::seek(uint position)
{
#ifdef _POSIX_MAPPED_FILES
  if(archive && position < bufferPosition)
#else // !_POSIX_MAPPED_FILES
  if(position < bufferPosition)
#endif
  {
    bufferPosition = position;
    this->position = 0;

    sys_seek(position);

    if(bufferPosition + bufferSize < fileSize)
      sys_read(buffer, bufferSize);
    else
      sys_read(buffer, fileSize - bufferPosition);

    return;
  }

  this->position = position - bufferPosition;
}

void File::put(uint8_t v)
{
  if(!bufferSize)
  {
    ::write(fd, &v, 1);

    return;
  }

  if(bufferPosition == bufferSize)
    flush();

  buffer[bufferPosition++] = v;
}

void File::put(uint16_t v)
{
  put(static_cast<uint8_t>(v));
  put(static_cast<uint8_t>(v >> 8));
}

void File::put(uint32_t v)
{
  put(static_cast<uint8_t>(v));
  put(static_cast<uint8_t>(v >> 8));
  put(static_cast<uint8_t>(v >> 16));
  put(static_cast<uint8_t>(v >> 24));
}

void File::put(const String& v)
{
  uint length = v.length();

  for(uint i = 0; i < length; ++i)
    put(static_cast<uint8_t>(v[i]));
}

void File::write(const void* buffer, uint count)
{
  if(bufferPosition)
    flush();

  ::write(fd, buffer, count);
}

void File::flush()
{
  ::write(fd, buffer, bufferPosition);

  bufferPosition = 0;
}

void File::skip(uint amount)
{
  seek(bufferPosition + position + amount);
}

bool File::eof() const
{
  return bufferPosition + position >= fileSize;
}

void File::sys_read(void* buffer, uint length)
{
#ifndef _POSIX_MAPPED_FILES
  if(archive)
#endif
  {
    archive->read(fd, buffer, length);
  }
#ifndef _POSIX_MAPPED_FILES
  else
  {
    ::read(fd, buffer, length);
  }
#endif
}

void File::sys_seek(uint position)
{
#ifndef _POSIX_MAPPED_FILES
  if(archive)
#endif
  {
    archive->seek(fd, position);
  }
#ifndef _POSIX_MAPPED_FILES
  else
  {
    ::lseek(fd, position, SEEK_SET);
  }
#endif
}

// vim: ts=2 sw=2 et
