/***************************************************************************
                            image.cc  -  OpenGL image class
                               -------------------
      copyright            : (C) 2003 by Morten Hustveit
      email                : morten@debian.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <map>

#include <string.h>

#include <espace/file.h>
#include <espace/image.h>
#include <espace/shader.h>
#include <espace/string.h>
#include <espace/opengl.h>
#include <espace/output.h>
#include <espace/plugins.h>

class Image::StaticInternal
{
public:

  std::map<String, Image*> handles;
};

Image::StaticInternal*       Image::s = new StaticInternal;

uint Image::acquire(const char* _name, bool mipmaps, bool linear, bool keep)
{
  String name(_name);

  if(name == "$lightmap")
    return lightmap;

  std::map<String, Image*>::iterator i = s->handles.find(name);

  if(i != s->handles.end())
  {
    ++i->second->refCount;

    return i->second->GLHandle();
  }

  String fileName = File::findSingle(name);

  if(fileName.isNull())
  {
    String base = name.left(name.length() - 4);
    String ext = name.right(4).toLower();

    if(ext == ".tga")
    {
      fileName = File::findSingle(base + ".jpg");
    }
    else if(ext == ".jpg")
    {
      fileName = File::findSingle(base + ".tga");
    }
    else // No extension
    {
      fileName = File::findSingle(name + ".tga");

      if(fileName.isNull())
        fileName = File::findSingle(name + ".jpg");
    }

    if(fileName.isNull())
      return 0;
  }

  File file(fileName);

  if(!file.isOpen())
  {
    esWarning << "Image: Failed to open \"" << name << "\"." << std::endl;

    return 0;
  }

  Image* image = 0;

  for(PluginMap(Image)::iterator i = Plugin::image.begin();
      i != Plugin::image.end(); ++i)
  {
    if(i->second->canHandle(file))
    {
      image = i->second->read(file);

      if(!image)
        continue;

      image->refCount = 1;

      break;
    }
  }

  if(!image)
  {
    esWarning << "Image: No plugin found for \"" << name << "\"." << std::endl;

    return 0;
  }

  image->setMipMaps(mipmaps);
  image->setLinear(linear);

  uint w = 1, h = 1;

  while(w < image->width() && w < GL::config.maxTextureSize)
    w <<= 1;

  while(h < image->height() && h < GL::config.maxTextureSize)
    h <<= 1;

  if(image->width() != w || image->height() != h)
    image->resize(w, h);

  image->upload();

  // XXX: I think this crashes with certain drivers.
  if(!keep)
  {
    delete [] image->_dataProxy->_data;

    image->_dataProxy->_data = 0;
  }

  s->handles[name] = image;

  return image->GLHandle();
}

Image* Image::acquireSimple(const char* _name)
{
  String name(_name);

  String fileName = File::findSingle(name);

  if(fileName.isNull())
  {
    String base = name.left(name.length() - 4);
    String ext = name.right(4).toLower();

    if(ext == ".tga")
    {
      fileName = File::findSingle(base + ".jpg");
    }
    else if(ext == ".jpg")
    {
      fileName = File::findSingle(base + ".tga");
    }
    else // No extension
    {
      fileName = File::findSingle(name + ".tga");

      if(fileName.isNull())
        fileName = File::findSingle(name + ".jpg");
    }

    if(fileName.isNull())
      return 0;
  }

  File file(fileName);

  if(!file.isOpen())
  {
    esWarning << "Image: Failed to open \"" << name << "\"." << std::endl;

    return 0;
  }

  Image* image = 0;

  for(PluginMap(Image)::iterator i = Plugin::image.begin();
      i != Plugin::image.end(); ++i)
  {
    if(i->second->canHandle(file))
    {
      image = i->second->read(file);

      if(!image)
        continue;

      image->refCount = 1;

      break;
    }
  }

  if(!image)
  {
    esWarning << "Image: No plugin found for \"" << name << "\"." << std::endl;

    return 0;
  }

  return image;
}

void Image::unacquire(uint handle)
{
  if(handle == lightmap)
    return;

  for(std::map<String, Image*>::iterator i = s->handles.begin();
      i != s->handles.end(); ++i)
  {
    if(i->second->GLHandle() == handle)
    {
      if(--i->second->refCount)
        return;

      delete i->second;

      s->handles.erase(i);

      return;
    }
  }
}

void Image::upload()
{
  if(!data())
    return;

  bool first = false;

  if(!GLHandle())
  {
    GL::genTextures(1, &_dataProxy->_GLHandle);

    first = true;
  }

  GL::bindTexture(GL::TEXTURE_2D, _dataProxy->_GLHandle);

  Shader::st_texture = _dataProxy->_GLHandle;

  if(_mipMaps)
  {
    if(_linear)
    {
      GL::texParameteri(GL::TEXTURE_2D, GL::TEXTURE_MAG_FILTER,
                        GL::LINEAR);
      GL::texParameteri(GL::TEXTURE_2D, GL::TEXTURE_MIN_FILTER,
                        GL::LINEAR_MIPMAP_NEAREST);
    }
    else // !_linear
    {
      GL::texParameteri(GL::TEXTURE_2D, GL::TEXTURE_MAG_FILTER,
                        GL::NEAREST);
      GL::texParameteri(GL::TEXTURE_2D, GL::TEXTURE_MIN_FILTER,
                        GL::NEAREST_MIPMAP_NEAREST);
    }
  }
  else if(_linear)
  {
    GL::texParameteri(GL::TEXTURE_2D, GL::TEXTURE_MAG_FILTER,
                      GL::LINEAR);
    GL::texParameteri(GL::TEXTURE_2D, GL::TEXTURE_MIN_FILTER,
                      GL::LINEAR);
  }
  else // !_linear && !_mipmaps
  {
    GL::texParameteri(GL::TEXTURE_2D, GL::TEXTURE_MAG_FILTER,
                      GL::NEAREST);
    GL::texParameteri(GL::TEXTURE_2D, GL::TEXTURE_MIN_FILTER,
                      GL::NEAREST);
  }

  GL::texParameteri(GL::TEXTURE_2D, GL::TEXTURE_WRAP_S,
                    _XRepeat ? GL::REPEAT : GL::CLAMP);

  GL::texParameteri(GL::TEXTURE_2D, GL::TEXTURE_WRAP_T,
                    _YRepeat ? GL::REPEAT : GL::CLAMP);

  if(first)
  {
    GL::texImage2D(
      GL::TEXTURE_2D,
      0,
      componentCount(),
      width(), height(),
      0,
        (pixelFormat() == RGB)    ? GL::RGB
      : (pixelFormat() == RGB16)  ? GL::RGB
      : (pixelFormat() == RGBA)   ? GL::RGBA
      : (pixelFormat() == RGBA16) ? GL::RGBA
      : (pixelFormat() == Gray)   ? GL::LUMINANCE
      : (pixelFormat() == Gray16) ? GL::LUMINANCE
      : 0,
      dataType(),
      data());
  }
  else // !first
  {
    GL::texSubImage2D(
      GL::TEXTURE_2D,
      0,
      0, 0, // Offset
      width(), height(),
        (pixelFormat() == RGB)    ? GL::RGB
      : (pixelFormat() == RGB16)  ? GL::RGB
      : (pixelFormat() == RGBA)   ? GL::RGBA
      : (pixelFormat() == RGBA16) ? GL::RGBA
      : (pixelFormat() == Gray)   ? GL::LUMINANCE
      : (pixelFormat() == Gray16) ? GL::LUMINANCE
      : 0,
      dataType(),
      data());
  }

  if((width() > 1 || height() > 1) && mipMaps())
  {
    uint componentCount = this->componentCount();
    uint mipLevel = 1;
    uint width = this->width();
    uint height = this->height();

    if(dataType() == GL::UNSIGNED_BYTE)
    {
      uint8_t* data = new uint8_t[width * height * componentCount / 4];

      while(width > 1 || height > 1)
      {
        uint oldWidth = width;

        uint XScale = (width > 1) ? 2 : 1;
        uint YScale = (height > 1) ? 2 : 1;

        width /= XScale;
        height /= YScale;

        const uint8_t* refData = (mipLevel == 1) ? this->data() : data;

        uint nextX = (XScale - 1);
        uint nextY = (YScale - 1);

        for(uint y = 0, n = 0; y < height; ++y)
          for(uint x = 0; x < width; ++x)
            for(uint c = 0; c < componentCount; ++c, ++n)
              data[n] =
                (refData[( y * YScale          * oldWidth +   x * XScale          ) * componentCount + c]
               + refData[( y * YScale          * oldWidth + ((x * XScale) + nextX)) * componentCount + c]
               + refData[((y * YScale + nextY) * oldWidth +   x * XScale          ) * componentCount + c]
               + refData[((y * YScale + nextY) * oldWidth + ((x * XScale) + nextX)) * componentCount + c]) / 4;

        GL::texImage2D(
          GL::TEXTURE_2D,
          mipLevel,
          componentCount,
          width, height,
          0,
            (pixelFormat() == RGB)    ? GL::RGB
          : (pixelFormat() == RGB16)  ? GL::RGB
          : (pixelFormat() == RGBA)   ? GL::RGBA
          : (pixelFormat() == RGBA16) ? GL::RGBA
          : (pixelFormat() == Gray)   ? GL::LUMINANCE
          : (pixelFormat() == Gray16) ? GL::LUMINANCE
          : 0,
          dataType(),
          data);

        ++mipLevel;
      }

      delete [] data;
    }
    else if(dataType() == GL::UNSIGNED_SHORT)
    {
      uint16_t* data = new uint16_t[width * height * componentCount / 4];

      while(width > 1 || height > 1)
      {
        uint oldWidth = width;

        uint XScale = (width > 1) ? 2 : 1;
        uint YScale = (height > 1) ? 2 : 1;

        width /= XScale;
        height /= YScale;

        const uint16_t* refData = (mipLevel == 1) ? this->data16() : data;

        uint nextX = (XScale - 1);
        uint nextY = (YScale - 1);

        for(uint y = 0, n = 0; y < height; ++y)
          for(uint x = 0; x < width; ++x)
            for(uint c = 0; c < componentCount; ++c, ++n)
              data[n] =
                 (refData[( y * YScale          * oldWidth +   x * XScale          ) * componentCount + c]
               +  refData[( y * YScale          * oldWidth + ((x * XScale) + nextX)) * componentCount + c]) / 2
               + (refData[((y * YScale + nextY) * oldWidth +   x * XScale          ) * componentCount + c]
               +  refData[((y * YScale + nextY) * oldWidth + ((x * XScale) + nextX)) * componentCount + c]) / 2;

        GL::texImage2D(
          GL::TEXTURE_2D,
          mipLevel,
          componentCount,
          width, height,
          0,
            (pixelFormat() == RGB16)  ? GL::RGB
          : (pixelFormat() == RGBA16) ? GL::RGBA
          : (pixelFormat() == Gray16) ? GL::LUMINANCE
          : 0,
          dataType(),
          data);

        ++mipLevel;
      }

      delete [] data;
    }
  }
}

void Image::setMipMaps(bool mipMaps)
{
  _mipMaps = mipMaps;

  if(GLHandle())
  {
    GL::bindTexture(GL::TEXTURE_2D, GLHandle());

    Shader::st_texture = _dataProxy->_GLHandle;

    // XXX: update
  }
}

void Image::setLinear(bool linear)
{
  _linear = linear;

  if(GLHandle())
  {
    GL::bindTexture(GL::TEXTURE_2D, GLHandle());

    Shader::st_texture = _dataProxy->_GLHandle;

    // XXX: update
  }
}

bool Image::mipMaps() const
{
  return _mipMaps;
}

bool Image::linear() const
{
  return _linear;
}

bool Image::XRepeat() const
{
  return _XRepeat;
}

bool Image::YRepeat() const
{
  return _YRepeat;
}

void Image::setRepeat(bool XRepeat, bool YRepeat)
{
  _XRepeat = XRepeat;
  _YRepeat = YRepeat;

  if(GLHandle())
  {
    GL::bindTexture(GL::TEXTURE_2D, GLHandle());

    Shader::st_texture = _dataProxy->_GLHandle;

    GL::texParameteri(GL::TEXTURE_2D, GL::TEXTURE_WRAP_S,
                      _XRepeat ? GL::REPEAT : GL::CLAMP);

    GL::texParameteri(GL::TEXTURE_2D, GL::TEXTURE_WRAP_T,
                      _YRepeat ? GL::REPEAT : GL::CLAMP);
  }
}

Image::PixelFormat Image::pixelFormat() const
{
  return _pixelFormat;
}

void Image::setPixelFormat(const Image::PixelFormat pixelFormat)
{
  _pixelFormat = pixelFormat;
}

uint Image::width() const
{
  return _width;
}

uint Image::height() const
{
  return _height;
}

double Image::aspect() const
{
  return static_cast<double>(_width) / _height;
}

uint Image::bytesPerPixel() const
{
  switch(_pixelFormat)
  {
  case RGB: return 3;
  case RGBA: return 4;
  case Gray: return 1;
  case RGB16: return 6;
  case RGBA16: return 8;
  case Gray16: return 2;
  default: return 0;
  }
}

uint Image::dataType() const
{
  switch(_pixelFormat)
  {
  case RGB:
  case RGBA:
  case Gray: return GL::UNSIGNED_BYTE;
  case RGB16:
  case RGBA16:
  case Gray16: return GL::UNSIGNED_SHORT;
  default: return 0;
  }
}

uint Image::componentCount() const
{
  switch(_pixelFormat)
  {
  case Gray:
  case Gray16: return 1;
  case RGB:
  case RGB16: return 3;
  case RGBA:
  case RGBA16: return 4;
  default: return 0;
  }
}

void Image::setGeometry(uint width, uint height)
{
  _width = width;
  _height = height;
}

void Image::setData(uint8_t* data)
{
  if(_dataProxy)
  {
    if(0 == --_dataProxy->_refCount)
      delete _dataProxy;
  }

  _dataProxy = new DataProxy();

  _dataProxy->_data = data;
}

uint8_t* Image::data()
{
  return _dataProxy ? _dataProxy->_data : 0;
}

uint16_t* Image::data16()
{
  if(dataType() != GL::UNSIGNED_BYTE)
    return 0;

  return reinterpret_cast<uint16_t*>(data());
}

const uint8_t* Image::data() const
{
  return _dataProxy ? _dataProxy->_data : 0;
}

void Image::brighten(float factor)
{
  uint bpp = bytesPerPixel();
  uint pixelCount = width() * height();
  uint8_t* c = data();
  uint32_t ffactor = static_cast<uint32_t>(factor * 0x100);

  switch(pixelFormat())
  {
  case Gray:
  case NV12:

    for(uint i = 0; i < pixelCount; ++i, ++c)
    {
      uint32_t i = *c * ffactor;

      *c = (i <= 0xFFFF) ? (i >> 8) : 0xFF;
    }

    break;

  case Gray16:

    for(uint i = 0; i < pixelCount; ++i, ++c)
    {
      uint32_t i = *c * ffactor;

      *c = (i <= 0xFFFFFF) ? (i >> 8) : 0xFFFF;
    }

    break;

  case RGB:
  case RGBA:
  case RGB16:
  case RGBA16:

    static uint32_t scale[256];
    static uint32_t lastffactor;

    if(ffactor != lastffactor)
    {
      for(uint i = 0; i < 256; ++i)
      {
        int j = i * ffactor;

        scale[i] = (j < 0x10000) ? ffactor : (0xFFFF / i);
      }

      lastffactor = ffactor;
    }

    for(uint i = 0; i < pixelCount; ++i, c += bpp)
    {
      uint32_t f = (c[0] > c[1] && c[0] > c[2]) ? (scale[c[0]])
                 : (c[1] > c[2])                ? (scale[c[1]])
                 :                                (scale[c[2]]);

      c[0] = (c[0] * f) >> 8;
      c[1] = (c[1] * f) >> 8;
      c[2] = (c[2] * f) >> 8;
    }
  }
}

Image& Image::operator=(const Image& image)
{
  ++image._dataProxy->_refCount;

  if(_dataProxy)
  {
    if(0 == --_dataProxy->_refCount)
      delete _dataProxy;
  }

  _dataProxy = image._dataProxy;

  setGeometry(image.width(), image.height());
  setPixelFormat(image.pixelFormat());

  return *this;
}

void Image::copy(const Image& image)
{
  if(image.width() != width()
  || image.height() != height())
  {
    esWarning << "Image: Resizing while copying unsupported." << std::endl;

    return;
  }

  if(image.pixelFormat() == pixelFormat())
  {
    memcpy(data(), image.data(), size());

    return;
  }

  switch(image.pixelFormat())
  {
  case Gray:

    switch(pixelFormat())
    {
    case NV12:

      memcpy(data(), image.data(), width() * height());
      memset(data() + width() * height(), 0, (height() / 2) * width());

    default:

      esWarning << "Image: Unsupported conversion." << std::endl;
    }

    break;

  case NV12:

    switch(pixelFormat())
    {
    case RGB:

      {
        uint8_t* RGB = data();
        const uint8_t* Y = image.data();
        uint width = this->width();
        uint height = this->height();
        const uint8_t* chroma = Y + width * height;

        for(uint y = 0; y < height; ++y)
        {
          const uint8_t* C = chroma + y / 2 * width;

          for(uint x = 0; x < width; x += 2, Y += 2, C += 2, RGB += 6)
          {
            int Cb = C[0] - 128;
            int Cr = C[1] - 128;

            int cRed   =  ((static_cast<int32_t>(0x10000 * 1.40200) * Cr) >> 16);
            int cGreen = -((static_cast<int32_t>(0x10000 * 0.34414) * Cb)
                         + (static_cast<int32_t>(0x10000 * 0.71414) * Cr) >> 16);
            int cBlue  =  ((static_cast<int32_t>(0x10000 * 1.77200) * Cb) >> 16);

            int r0 = Y[0] + cRed;
            int g0 = Y[0] + cGreen;
            int b0 = Y[0] + cBlue;
            int r1 = Y[1] + cRed;
            int g1 = Y[1] + cGreen;
            int b1 = Y[1] + cBlue;

            RGB[0] = (r0 >= 255) ? 255 : (r0 <= 0) ? 0 : r0;
            RGB[1] = (g0 >= 255) ? 255 : (g0 <= 0) ? 0 : g0;
            RGB[2] = (b0 >= 255) ? 255 : (b0 <= 0) ? 0 : b0;
            RGB[3] = (r1 >= 255) ? 255 : (r1 <= 0) ? 0 : r1;
            RGB[4] = (g1 >= 255) ? 255 : (g1 <= 0) ? 0 : g1;
            RGB[5] = (b1 >= 255) ? 255 : (b1 <= 0) ? 0 : b1;
          }
        }
      }

      break;

    default:

      esWarning << "Image: Unsupported conversion." << std::endl;
    }

    break;

  default:

    esWarning << "Image::Unsupported conversion." << std::endl;
  }
}

void Image::resize(uint width, uint height)
{
  uint oldWidth = this->width();
  uint oldHeight = this->height();
  uint8_t* oldData = data();
  uint8_t* data = new uint8_t[width * height * bytesPerPixel()];

  uint stepX = (oldWidth * 0x10000) / width;
  uint stepY = (oldHeight * 0x10000) / height;

  uint bytesPerPixel = this->bytesPerPixel();

  // The following works poorly for downsampling

  if(!mipMaps() || true /* XXX */)
  {
    if(_linear)
    {
      for(uint y = 0, oldY = 0; y < height; ++y, oldY += stepY)
      {
        for(uint x = 0, oldX = 0; x < width; ++x, oldX += stepX)
        {
          for(uint n = 0; n < bytesPerPixel; ++n)
          {
            data[(y * width + x) * bytesPerPixel + n] =
              oldData[((oldY >> 16) * oldWidth + (oldX >> 16)) * bytesPerPixel + n];
          }
        }
      }
    }
    else // !_linear
    {
      for(uint y = 0, oldY = 0; y < height; ++y, oldY += stepY)
      {
        for(uint x = 0, oldX = 0; x < width; ++x, oldX += stepX)
        {
          for(uint n = 0; n < bytesPerPixel; ++n)
          {
            data[(y * width + x) * bytesPerPixel + n] =
              oldData[((int) ((float) oldHeight / height * y + 0.5) * oldWidth
                       + (int) ((float) oldWidth / width * x + 0.5))
                      * bytesPerPixel + n];
          }
        }
      }
    }
  }
  else // mipMaps()
  {
    // XXX: Implement me! :(
  }

  setGeometry(width, height);
  setData(data);
}

uint Image::size() const
{
  if(pixelFormat() == NV12)
    return (height() + height() / 2) * width();

  return bytesPerPixel() * width() * height();
}

Image::DataProxy::DataProxy()
  : _refCount(1),
    _data(0),
    _GLHandle(0)
{
}

Image::DataProxy::~DataProxy()
{
  if(_GLHandle)
    GL::deleteTextures(1, &_GLHandle);

  if(_data)
    delete [] _data;
}

// vim: ts=2 sw=2 et
