#ifndef _IMPORT_H_
#define _IMPORT_H_

#include <stdint.h>

#include "types.h"

/**
 * Class for holding image data, and optionally an OpenGL handle.
 * \author Morten Hustveit
 */
class Image
{
public:

  /**
   * Loads an image and creates an OpenGL handle for it.
   *
   * If the image's width or height is not a power of 2, the image is rescaled
   * to fit this requirement.
   *
   * If this function is called several times with the same parameters, no new
   * data is allocated, but a reference counter is increased instead.  The data
   * is not freed until unacquire() is called an equal amount of times on the
   * returned handle.
   *
   * Returns OpenGL handle for the image on success, zero on failure.
   *
   * \param name The file name of the image.
   * \param mipmaps Whether or not to generate mipmaps.
   * \param linear Whether or not to use linear filtering.
   * \param keep Whether or not to keep the image data in memory after uploading
   *             to OpenGL driver.
   *
   * \todo Use a single flag parameter instead of lots of booleans.
   */
  static IMPORT uint acquire(const char* name, bool mipmaps = true,
                             bool linear = true, bool keep = false);

  /**
   * Frees the memory allocated by the image.
   *
   * Nothing is freed unless the reference count reaches zero.
   *
   * \param handle Handle of the image to deallocate.
   */
  static IMPORT void unacquire(uint handle);

  /**
   * Reads an image.
   *
   * This function does not create a GL handle.  You can call the upload()
   * function to get a GL handle for the returned image.
   */
  static IMPORT Image* acquireSimple(const char* name);

  static const uint lightmap = ~0UL;

  enum PixelFormat
  {
    RGB = 1,
    RGBA = 2,
    Gray = 3,
    NV12 = 4, // See www.fourcc.org
    RGB16 = 5,
    RGBA16 = 6,
    Gray16 = 7
  };

  Image()
    : _XRepeat(true),
      _YRepeat(true),
      _mipMaps(true),
      _dataProxy(0),
      refCount(0)
  { }

  ~Image()
  {
    if(!_dataProxy)
      return;

    if(0 == --_dataProxy->_refCount)
      delete _dataProxy;
  }

  /**
   * Scales the image data to the specified resolution.
   */
  IMPORT void resize(uint width, uint height);

  /**
   * Copies the contents of this image into another previously allocated image.
   *
   * If the images are of different resolutions or pixel formats, the data is
   * converted.
   */
  IMPORT void copy(const Image&);

  /**
   * Uploads an image to the OpenGL driver.
   *
   * \see GLHandle()
   */
  IMPORT void upload();

  /**
   * Returns the OpenGL handle of this image.
   *
   * The handle is set by a call to upload().
   */
  uint GLHandle() const
  {
    return _dataProxy->_GLHandle;
  }

  /**
   * Returns the pixel format.
   */
  IMPORT PixelFormat pixelFormat() const;

  /**
   * Returns the width of in pixels.
   */
  IMPORT uint width() const;

  /**
   * Returns the height of in pixels.
   */
  IMPORT uint height() const;

  /**
   * Returns the amount of bytes per pixel.
   */
  IMPORT uint bytesPerPixel() const;

  /**
   * Returns the data type used for storing color components.
   *
   * This is either GL::UNSIGNED_BYTE or GL::UNSIGNED_SHORT
   */
  IMPORT uint dataType() const;

  /**
   * Returns the amount of color components.
   *
   * This is usually 1, 3 or 4.
   */
  IMPORT uint componentCount() const;

  /**
   * Returns the size of this image in bytes.
   */
  IMPORT uint size() const;

  /**
   * Creates another handle to this image.
   *
   * No data is copied, but a reference counter is kept instead.  When all
   * references to the data are destroyed, the data is freed.
   */
  IMPORT Image& operator=(const Image& image);

  // The following functions WILL change in the feature, and are therefore not
  // documented.
  IMPORT void setRepeat(bool XRepeat, bool YRepeat);
  IMPORT void setPixelFormat(const PixelFormat pixelFormat);
  IMPORT void setGeometry(uint width, uint height);
  IMPORT void setData(uint8_t* data);
  IMPORT void setMipMaps(bool mipMaps);
  IMPORT void setLinear(bool linear);
  IMPORT void brighten(float factor);

  IMPORT bool XRepeat() const;
  IMPORT bool YRepeat() const;
  IMPORT bool mipMaps() const;
  IMPORT bool linear() const;
  IMPORT double aspect() const;
  IMPORT uint8_t* data();
  IMPORT uint16_t* data16();
  IMPORT const uint8_t* data() const;

protected:

  class DataProxy
  {
  protected:

    friend class Image;

    DataProxy();
    IMPORT ~DataProxy();

    uint     _refCount;
    uint8_t* _data;
    uint     _GLHandle;
  };

  bool         _XRepeat, _YRepeat;
  bool         _mipMaps;
  bool         _linear;
  PixelFormat  _pixelFormat;	
  uint         _width, _height;
  DataProxy*   _dataProxy;

  uint         refCount;

  class StaticInternal;

  static StaticInternal* s;
};

#endif

// vim: ts=2 sw=2 et
