#ifndef SHADER_H_
#define SHADER_H_

#ifndef SWIG
#include "color.h"
#include "types.h"
#endif

/**
 * Shader routines.
 *
 * \author Morten Hustveit
 */
struct Shader
{
  /**
   * Initializes all shader plugins.
   *
   * The shader plugins will scan for installed shaders, register the ones they
   * can handle.
   */
  static IMPORT void initialize();

  /**
   * Acquire a handle for a shader.
   *
   * Returns NULL if the shader is not found.
   */
  static IMPORT Shader* acquire(const char* name, bool mipmaps = true);

  /**
   * Acquire an integer handle for a shader.
   *
   * Returns 0 if the shader is not found.
   */
  static IMPORT int acquireHandle(const char* name, bool mipmaps = true);

  /**
   * Unacquire a shader handle acquired by a call to acquire().
   */
  static IMPORT void unacquire(Shader*);

  /**
   * Unacquire an integer shader handle acquired by a call to acquireHandle().
   */
  static IMPORT void unacquire(int handle);

  /**
   * Converts an integer handle to a pointer handle.
   *
   * Returns NULL if the handle is invalid.
   */
  static IMPORT Shader* shaderForHandle(int handle);

  /**
   * Creates an alternate name for a shader.
   */
  static IMPORT void remap(const char* oldShader, const char* newShader);

  /**
   * Sets the default rendering parameters.
   *
   * This function is useful if the application may have messed up the OpenGL
   * state.
   */
  static IMPORT void setDefaults();

  static IMPORT void setVertices(const void* vertices, uint stride = 0);
  static IMPORT void setColors(const void* colors, uint stride = 0);
  static IMPORT void setTextureCoords(const void* coords, uint stride = 0);
  static IMPORT void setLightmapCoords(const void* coords, uint stride = 0);
  static IMPORT void setNormals(const void* normals, uint stride = 0);

  /**
   * Enter 2D rendering mode.
   *
   * This function maps space coordinates to window coordinates and disables the
   * depth buffer.  Useful for drawing graphical user interfaces.
   */
  static IMPORT void set2DMode();

  /**
   * Undo a call to set2DMode().
   */
  static IMPORT void set3DMode();

#ifndef SWIG
  virtual IMPORT ~Shader();

  virtual IMPORT void unacquire() = 0;

  virtual IMPORT uint passCount() const = 0;
  virtual IMPORT void pushState(uint pass) = 0;
  virtual IMPORT void popState() = 0;

  enum Face
  {
    Front,
    Back,
    None
  };

  virtual IMPORT bool isSky() const = 0;
  virtual IMPORT uint sort() const = 0;
  virtual IMPORT Face cullFace() const = 0;

  uint refCount;

  enum TexCoords
  {
    Texture, Lightmap, User
  };

  enum WaveForm
  {
    Sawtooth,
    InverseSawtooth,
    Sin,
    Square,
    Triangle
  };

  enum VertexDeform
  {
    Wave,
    Normal,
    Bulge,
    Move,
    AutoSprite,
    AutoSprite2
  };

  enum Sort
  {
    Portal =     (1 << 24),
    Sky =        (2 << 24),
    Opaque =     (3 << 24),
    Banner =     (6 << 24),
    Underwater = (8 << 24),
    Additive =   (9 << 24),
    Nearest =    (16 << 24)
  };

  enum AlphaFunc
  {
    All,
    GT0,
    LT128,
    GE128
  };

  static IMPORT float time;

  static IMPORT float wave(WaveForm wave, float base, float amp, float phase, float freq, float time);

  // This have to go

  static IMPORT bool      st_coloring;
  static IMPORT Face      st_cullFace;
  static IMPORT uint      st_sourceBlend;
  static IMPORT uint      st_destBlend;
  static IMPORT AlphaFunc st_alphaFunc;
  static IMPORT bool      st_polygonOffset;
  static IMPORT bool      st_depthWrite;
  static IMPORT uint      st_texture;
  static IMPORT bool      st_texturing;
  static IMPORT bool      st_vertexColors;
  static IMPORT bool      st_withAlpha;
  static IMPORT TexCoords st_texCoords;
  static IMPORT bool      st_tcMod;
  static IMPORT Color     st_entityColor;

  static IMPORT uint        vertexStride;
  static IMPORT const char* vertexPointer;
  static IMPORT uint        colorStride;
  static IMPORT const char* colorPointer;
  static IMPORT uint        texCoordStride;
  static IMPORT const char* texCoordPointer;
  static IMPORT uint        lightCoordStride;
  static IMPORT const char* lightCoordPointer;
  static IMPORT uint        normalStride;
  static IMPORT const char* normalPointer;
#endif // !SWIG
};

#endif // !SHADER_H_

// vim: ts=2 sw=2 et
