/***************************************************************************
                          montage.cc  -  Image montage routines
                               -------------------
      copyright            : (C) 2003 by Morten Hustveit
      email                : morten@debian.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <vector>

#include <espace/color.h>
#include <espace/image.h>
#include <espace/montage.h>
#include <espace/opengl.h>
#include <espace/shader.h>

Montage::Montage(Image* image)
  : width(0),
    height(0),
    images(0)
{
  uint w = image->width();
  uint h = image->height();

  std::vector<uint> widths;
  std::vector<uint> heights;

  while(w)
  {
    ++width;

    if(w >= GL::config.maxTextureSize)
    {
      widths.push_back(GL::config.maxTextureSize);

      w -= GL::config.maxTextureSize;
    }
    else
    {
      uint i = 1;

      while(i * 2 <= w)
        i *= 2;

      widths.push_back(i);

      w -= i;
    }
  }

  while(h)
  {
    ++height;

    if(h >= GL::config.maxTextureSize)
    {
      heights.push_back(GL::config.maxTextureSize);

      h -= GL::config.maxTextureSize;
    }
    else
    {
      uint i = 1;

      while(i * 2 <= h)
        i *= 2;

      heights.push_back(i);

      h -= i;
    }
  }

  images = new Image*[width * height];

  uint8_t* sourceData = image->data();
  uint     bpp = image->bytesPerPixel();
  uint     sourceWidth = image->width();

  for(uint y = 0; y < height; ++y)
  {
    w = 0;

    for(uint x = 0; x < width; ++x)
    {
      Image* subImage = images[y * width + x] = new Image;

      subImage->setGeometry(widths[x], heights[y]);
      subImage->setPixelFormat(image->pixelFormat());

      uint8_t* data = new uint8_t[widths[x] * heights[y] * bpp];

      for(uint i = 0; i < heights[y]; ++i)
      {
        memcpy(data + i * widths[x] * bpp,
               sourceData + ((i + h) * sourceWidth + w) * bpp,
               widths[x] * bpp);
      }

      subImage->setData(data);
      subImage->setRepeat(false, false);
      subImage->setMipMaps(false);
      subImage->setLinear(false);
      subImage->upload();

      w += widths[x];
    }

    h += heights[y];
  }

  sourceBlend = GL::SRC_ALPHA;
  destBlend = GL::ONE_MINUS_SRC_ALPHA;
}

Montage::~Montage()
{
  for(uint i = 0; i < width * height; ++i)
    delete images[i];

  delete [] images;
}

void Montage::render(float __x, float _y) const
{
  if(!Shader::st_coloring)
  {
    GL::texEnvi(GL::TEXTURE_ENV, GL::TEXTURE_ENV_MODE, GL::MODULATE);

    Shader::st_coloring = true;
  }

  if(Shader::st_sourceBlend != sourceBlend
  || Shader::st_destBlend != destBlend)
  {
    GL::blendFunc(sourceBlend, destBlend);

    Shader::st_sourceBlend = sourceBlend;
    Shader::st_sourceBlend = destBlend;
  }

  GL::color4ubv(Shader::st_entityColor.data());

  if(Shader::st_cullFace != Shader::Back)
  {
    GL::enable(GL::CULL_FACE);
    GL::cullFace(GL::BACK);

    Shader::st_cullFace = Shader::Back;
  }

  for(uint y = 0; y < height; ++y)
  {
    float _x = __x;

    for(uint x = 0; x < width; ++x)
    {
      Image* image = images[y * width + x];

      Shader::st_texture = image->GLHandle();
      GL::bindTexture(GL::TEXTURE_2D, image->GLHandle());

      GL::begin(GL::QUADS);
      GL::texCoord2f(0, 0);
      GL::vertex2f(_x, _y);
      GL::texCoord2f(0, 1);
      GL::vertex2f(_x, _y + image->height());
      GL::texCoord2f(1, 1);
      GL::vertex2f(_x + image->width(), _y + image->height());
      GL::texCoord2f(1, 0);
      GL::vertex2f(_x + image->width(), _y);
      GL::end();

      _x += image->width();

      if(x == width - 1)
        _y += image->height();
    }
  }
}

void Montage::setSourceBlend(uint factor)
{
  sourceBlend = factor;
}

void Montage::setDestBlend(uint factor)
{
  destBlend = factor;
}

// vim: ts=2 sw=2 et
