/***************************************************************************
                           openal.cc  -  OpenAL function declarations
                               -------------------
      copyright            : (C) 2003 by Morten Hustveit
      email                : morten@debian.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <espace/cvar.h>
#include <espace/output.h>
#include <espace/system.h>

#include "openal.h"

namespace
{
  bool _initialized = false;

  typedef void (*alutInit)(int *argc, char *argv[]);
  typedef void (*alutExit)(ALvoid);

  alutInit utInit;
  alutExit utExit;

  void uninitialize()
  {
    _initialized = false;

    utExit();
  }
}

void AL::initialize()
{
  if(_initialized)
    return;

  CVar alDriver = CVar::acquire("s_aldriver",
#ifdef WIN32
                               "openal32.dll",
#else
                               "libopenal.so.0",
#endif
                               CVar::Archive);

  void* library = System::dlopen(alDriver.string);

  if(!library)
  {
    esWarning << "Failed to load OpenAL library \"" << alDriver.string << "\": "
              << System::dlerror() << std::endl;

    return;
  }

  enable = (alEnable) System::dlsym(library, "alEnable");
  disable = (alDisable) System::dlsym(library, "alDisable");
  isEnabled = (alIsEnabled) System::dlsym(library, "alIsEnabled");
  hint = (alHint) System::dlsym(library, "alHint");
  getBoolean = (alGetBoolean) System::dlsym(library, "alGetBoolean");
  getInteger = (alGetInteger) System::dlsym(library, "alGetInteger");
  getFloat = (alGetFloat) System::dlsym(library, "alGetFloat");
  getDouble = (alGetDouble) System::dlsym(library, "alGetDouble");
  getBooleanv = (alGetBooleanv) System::dlsym(library, "alGetBooleanv");
  getIntegerv = (alGetIntegerv) System::dlsym(library, "alGetIntegerv");
  getFloatv = (alGetFloatv) System::dlsym(library, "alGetFloatv");
  getDoublev = (alGetDoublev) System::dlsym(library, "alGetDoublev");
  getString = (alGetString) System::dlsym(library, "alGetString");
  getError = (alGetError) System::dlsym(library, "alGetError");
  isExtensionPresent = (alIsExtensionPresent) System::dlsym(library, "alIsExtensionPresent");
  getProcAddress = (alGetProcAddress) System::dlsym(library, "alGetProcAddress");
  getEnumValue = (alGetEnumValue) System::dlsym(library, "alGetEnumValue");

  listenerf = (alListenerf) System::dlsym(library, "alListenerf");
  listenerfv = (alListenerfv) System::dlsym(library, "alListenerfv");
  getListeneri = (alGetListeneri) System::dlsym(library, "alGetListeneri");
  getListenerf = (alGetListenerf) System::dlsym(library, "alGetListenerf");
  getListeneriv = (alGetListeneriv) System::dlsym(library, "alGetListeneriv");
  getListenerfv = (alGetListenerfv) System::dlsym(library, "alGetListenerfv");

  genSources = (alGenSources) System::dlsym(library, "alGenSources");
  deleteSources = (alDeleteSources) System::dlsym(library, "alDeleteSources");
  isSource = (alIsSource) System::dlsym(library, "alIsSource");
  sourcei = (alSourcei) System::dlsym(library, "alSourcei");
  sourcef = (alSourcef) System::dlsym(library, "alSourcef");
  source3f = (alSource3f) System::dlsym(library, "alSource3f");
  sourcefv = (alSourcefv) System::dlsym(library, "alSourcefv");
  getSourcei = (alGetSourcei) System::dlsym(library, "alGetSourcei");
  getSourceiv = (alGetSourceiv) System::dlsym(library, "alGetSourceiv");
  getSourcef = (alGetSourcef) System::dlsym(library, "alGetSourcef");
  getSource3f = (alGetSource3f) System::dlsym(library, "alGetSource3f");
  getSourcefv = (alGetSourcefv) System::dlsym(library, "alGetSourcefv");
  sourcePlay = (alSourcePlay) System::dlsym(library, "alSourcePlay");
  sourcePause = (alSourcePause) System::dlsym(library, "alSourcePause");
  sourceStop = (alSourceStop) System::dlsym(library, "alSourceStop");
  sourceRewind = (alSourceRewind) System::dlsym(library, "alSourceRewind");
  sourcePlayv = (alSourcePlayv) System::dlsym(library, "alSourcePlayv");
  sourcePausev = (alSourcePausev) System::dlsym(library, "alSourcePausev");
  sourceStopv = (alSourceStopv) System::dlsym(library, "alSourceStopv");
  sourceRewindv = (alSourceRewindv) System::dlsym(library, "alSourceRewindv");

  genBuffers = (alGenBuffers) System::dlsym(library, "alGenBuffers");
  deleteBuffers = (alDeleteBuffers) System::dlsym(library, "alDeleteBuffers");
  isBuffer = (alIsBuffer) System::dlsym(library, "alIsBuffer");
  bufferData = (alBufferData) System::dlsym(library, "alBufferData");
  getBufferi = (alGetBufferi) System::dlsym(library, "alGetBufferi");
  getBufferf = (alGetBufferf) System::dlsym(library, "alGetBufferf");
  getBufferiv = (alGetBufferiv) System::dlsym(library, "alGetBufferiv");
  getBufferfv = (alGetBufferfv) System::dlsym(library, "alGetBufferfv");

  queuei = (alQueuei) System::dlsym(library, "alQueuei");
  sourceUnqueueBuffers = (alSourceUnqueueBuffers) System::dlsym(library, "alSourceUnqueueBuffers");
  sourceQueueBuffers = (alSourceQueueBuffers) System::dlsym(library, "alSourceQueueBuffers");

  utInit = (alutInit) System::dlsym(library, "alutInit");
  utExit = (alutExit) System::dlsym(library, "alutExit");

  utInit(0, 0);

  atexit(uninitialize);

  _initialized = true;
}

bool AL::initialized()
{
  return _initialized;
}

AL::alEnable             AL::enable;
AL::alDisable            AL::disable;
AL::alIsEnabled          AL::isEnabled;
AL::alHint               AL::hint;
AL::alGetBoolean         AL::getBoolean;
AL::alGetInteger         AL::getInteger;
AL::alGetFloat           AL::getFloat;
AL::alGetDouble          AL::getDouble;
AL::alGetBooleanv        AL::getBooleanv;
AL::alGetIntegerv        AL::getIntegerv;
AL::alGetFloatv          AL::getFloatv;
AL::alGetDoublev         AL::getDoublev;
AL::alGetString          AL::getString;
AL::alGetError           AL::getError;
AL::alIsExtensionPresent AL::isExtensionPresent;
AL::alGetProcAddress     AL::getProcAddress;
AL::alGetEnumValue       AL::getEnumValue;

AL::alListenerf          AL::listenerf;
AL::alListenerfv         AL::listenerfv;
AL::alGetListeneri       AL::getListeneri;
AL::alGetListenerf       AL::getListenerf;
AL::alGetListeneriv      AL::getListeneriv;
AL::alGetListenerfv      AL::getListenerfv;

AL::alGenSources         AL::genSources;
AL::alDeleteSources      AL::deleteSources;
AL::alIsSource           AL::isSource;
AL::alSourcei            AL::sourcei;
AL::alSourcef            AL::sourcef;
AL::alSource3f           AL::source3f;
AL::alSourcefv           AL::sourcefv;
AL::alGetSourcei         AL::getSourcei;
AL::alGetSourceiv        AL::getSourceiv;
AL::alGetSourcef         AL::getSourcef;
AL::alGetSource3f        AL::getSource3f;
AL::alGetSourcefv        AL::getSourcefv;
AL::alSourcePlay         AL::sourcePlay;
AL::alSourcePause        AL::sourcePause;
AL::alSourceStop         AL::sourceStop;
AL::alSourceRewind       AL::sourceRewind;
AL::alSourcePlayv        AL::sourcePlayv;
AL::alSourcePausev       AL::sourcePausev;
AL::alSourceStopv        AL::sourceStopv;
AL::alSourceRewindv      AL::sourceRewindv;

AL::alGenBuffers         AL::genBuffers;
AL::alDeleteBuffers      AL::deleteBuffers;
AL::alIsBuffer           AL::isBuffer;
AL::alBufferData         AL::bufferData;
AL::alGetBufferi         AL::getBufferi;
AL::alGetBufferf         AL::getBufferf;
AL::alGetBufferiv        AL::getBufferiv;
AL::alGetBufferfv        AL::getBufferfv;

AL::alQueuei               AL::queuei;
AL::alSourceUnqueueBuffers AL::sourceUnqueueBuffers;
AL::alSourceQueueBuffers   AL::sourceQueueBuffers;

// vim: ts=2 sw=2 et
