/***************************************************************************
                     simpleshader.cc  -  Simple shader handler
                               -------------------
      copyright            : (C) 2003 by Morten Hustveit
      email                : morten@debian.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <espace/color.h>
#include <espace/file.h>
#include <espace/image.h>
#include <espace/opengl.h>
#include <espace/shader.h>

#include "simpleshader.h"

class SimpleShaderData : public Shader
{
public:

  SimpleShaderData(uint image);

  void unacquire();

  uint passCount() const;
  void pushState(uint pass);
  void popState();

  bool isSky() const;
  uint sort() const;
  Face cullFace() const;

protected:

  uint image;
};

uint32_t SimpleShader::id()
{
  return 0x534D504C; // 'SMPL'
}

bool SimpleShader::canHandle(File&)
{
  return false;
}

void SimpleShader::initialize()
{
}

Shader* SimpleShader::acquire(const char* _name, bool mipmaps)
{
  String name(_name);

  bool linear;

  if(name.beginsWith("simple:"))
  {
    linear = true;
  }
  else if(name.beginsWith("simple-nolinear:"))
  {
    linear = false;
  }
  else
    return 0;

  name = name.right(name.length() - name.find(":") - 1);

  uint image;

  if(name == "*white")
  {
    image = 0;
  }
  else
  {
    image = Image::acquire(name, mipmaps, linear);

    if(!image)
      return 0;
  }

  SimpleShaderData* shader = new SimpleShaderData(image);

  return shader;
}

SimpleShaderData::SimpleShaderData(uint image)
  : image(image)
{
  refCount = 1;
}

void SimpleShaderData::unacquire()
{
  if(image)
    Image::unacquire(image);
}

uint SimpleShaderData::passCount() const
{
  return 1;
}

void SimpleShaderData::pushState(uint pass)
{
  if(!st_coloring)
  {
    GL::texEnvi(GL::TEXTURE_ENV, GL::TEXTURE_ENV_MODE, GL::MODULATE);

    st_coloring = true;
  }

  GL::color4ubv(st_entityColor.data());

  if(!st_texturing)
  {
    GL::enableClientState(GL::TEXTURE_COORD_ARRAY);

    st_texturing = true;
  }

  if(st_texCoords != Texture)
  {
    GL::texCoordPointer(2, GL::FLOAT, texCoordStride, texCoordPointer);

    st_texCoords = Texture;
  }

  if(st_texture != image)
  {
    GL::bindTexture(GL::TEXTURE_2D, image);

    st_texture = image;
  }

  if(st_sourceBlend != GL::SRC_ALPHA
  || st_destBlend != GL::ONE_MINUS_SRC_ALPHA)
  {
    GL::blendFunc(GL::SRC_ALPHA, GL::ONE_MINUS_SRC_ALPHA);

    st_sourceBlend = GL::SRC_ALPHA;
    st_destBlend = GL::ONE_MINUS_SRC_ALPHA;
  }

  if(st_cullFace != Back)
  {
    GL::enable(GL::CULL_FACE);
    GL::cullFace(GL::BACK);

    st_cullFace = Back;
  }

  if(st_alphaFunc != All)
  {
    GL::disable(GL::ALPHA_TEST);

    st_alphaFunc = All;
  }

  if(st_polygonOffset)
  {
    GL::disable(GL::POLYGON_OFFSET_FILL);

    st_polygonOffset = false;
  }

  if(st_vertexColors)
  {
    GL::disableClientState(GL::COLOR_ARRAY);

    st_vertexColors = false;
  }

  if(!st_depthWrite)
  {
    GL::depthMask(GL::TRUE);

    st_depthWrite = true;
  }
}

void SimpleShaderData::popState()
{
}

bool SimpleShaderData::isSky() const
{
  return false;
}

uint SimpleShaderData::sort() const
{
  return 0;
}

Shader::Face SimpleShaderData::cullFace() const
{
  return Back;
}

// vim: ts=2 sw=2 et
