/***************************************************************************
                           shader.cc  -  Shader routines
                               -------------------
      copyright            : (C) 2003 by Morten Hustveit
      email                : morten@debian.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <map>
#include <math.h>

#include <espace/color.h>
#include <espace/file.h>
#include <espace/opengl.h>
#include <espace/plugins.h>
#include <espace/shader.h>
#include <espace/string.h>

static std::map<int, Shader*>   handles;
static std::map<String, int>    reverseHandles;
static std::map<String, String> remapping;
static int                      nextHandle = 1;
static bool                     mode2D = false;

bool              Shader::st_coloring = false;
Shader::Face      Shader::st_cullFace = Front;
uint              Shader::st_sourceBlend = GL::ONE;
uint              Shader::st_destBlend = GL::ZERO;
Shader::AlphaFunc Shader::st_alphaFunc = Shader::All;
bool              Shader::st_polygonOffset = false;
bool              Shader::st_depthWrite = true;
uint              Shader::st_texture = 0;
bool              Shader::st_texturing = true;
bool              Shader::st_vertexColors = false;
bool              Shader::st_withAlpha = false;
Shader::TexCoords Shader::st_texCoords = Shader::User;
bool              Shader::st_tcMod = false;
Color             Shader::st_entityColor(255, 255, 255);

uint              Shader::vertexStride = 0;
const char*       Shader::vertexPointer = 0;
uint              Shader::colorStride = 0;
const char*       Shader::colorPointer = 0;
uint              Shader::texCoordStride = 0;
const char*       Shader::texCoordPointer = 0;
uint              Shader::lightCoordStride = 0;
const char*       Shader::lightCoordPointer = 0;
uint              Shader::normalStride = 0;
const char*       Shader::normalPointer = 0;
float             Shader::time = 0;

void Shader::initialize()
{
  std::map<uint, ShaderPlugin*>::iterator i;

  for(i = Plugin::shader.begin(); i != Plugin::shader.end(); ++i)
    i->second->initialize();
}

Shader* Shader::acquire(const char* _name, bool mipmaps)
{
  String name(_name);

  std::map<String, String>::iterator j = remapping.find(name);

  if(j != remapping.end())
    name = j->second;

  std::map<String, int>::iterator i = reverseHandles.find(name);

  if(i != reverseHandles.end())
  {
    if(i->second == 0)
      return 0;

    Shader* shader = handles[i->second];

    ++shader->refCount;

    return shader;
  }

  for(std::map<uint, ShaderPlugin*>::iterator i = Plugin::shader.begin();
      i != Plugin::shader.end(); ++i)
  {
    Shader* shader = i->second->acquire(name, mipmaps);

    if(shader)
    {
      handles[nextHandle++] = shader;
      reverseHandles[name] = nextHandle - 1;

      return shader;
    }
  }

  reverseHandles[name] = 0;

  return 0;
}

int Shader::acquireHandle(const char* _name, bool mipmaps)
{
  String name(_name);

  std::map<String, String>::iterator j = remapping.find(name);

  if(j != remapping.end())
    name = j->second;

  std::map<String, int>::iterator i = reverseHandles.find(name);

  if(i != reverseHandles.end())
  {
    if(i->second == 0)
      return 0;

    Shader* shader = handles[i->second];

    ++shader->refCount;

    return i->second;
  }

  for(std::map<uint, ShaderPlugin*>::iterator i = Plugin::shader.begin();
      i != Plugin::shader.end(); ++i)
  {
    Shader* shader = i->second->acquire(name, mipmaps);

    if(shader)
    {
      handles[nextHandle++] = shader;
      reverseHandles[name] = nextHandle - 1;

      return nextHandle - 1;
    }
  }

  reverseHandles[name] = 0;

  return 0;
}

void Shader::unacquire(Shader* shader)
{
  if(--shader->refCount)
    return;

  for(std::map<int, Shader*>::iterator i = handles.begin();
      i != handles.end(); ++i)
  {
    if(i->second == shader)
    {
      handles.erase(i);

      for(std::map<String, int>::iterator j = reverseHandles.begin();
         j != reverseHandles.end(); ++j)
      {
        if(j->second == i->first)
          reverseHandles.erase(j);
      }

      break;
    }
  }

  shader->unacquire();
}

void Shader::unacquire(int handle)
{
  Shader* shader = shaderForHandle(handle);

  unacquire(shader);
}

Shader* Shader::shaderForHandle(int handle)
{
  std::map<int, Shader*>::iterator i = handles.find(handle);

  if(i == handles.end())
    return 0;

  return i->second;
}

void Shader::remap(const char* oldShader, const char* newShader)
{
  remapping[oldShader] = newShader;
}

float Shader::wave(WaveForm wave, float base, float amp, float phase,
                   float freq, float time)
{
  float value = base;
  float pos = phase + freq * time;
  float subpos = fmod(pos, 1);

  switch(wave)
  {
  case Sawtooth:

    value += amp * subpos;

    break;

  case InverseSawtooth:

    value += amp * (1 - subpos);

    break;

  case Sin:

    value += amp * sin(pos * 2 * M_PI);

    break;

  case Square:

    value += (subpos < 0.5) ? amp : -amp;

    break;

  case Triangle:

    value += amp * ((subpos < 0.5) ? (2 * subpos) : (2 - 2 * subpos));

    break;
  }

  return (value > 1) ? 1
       : (value < 0) ? 0
       :               value;
}

void Shader::setDefaults()
{
  GL::enable(GL::SCISSOR_TEST);

  GL::enable(GL::DEPTH_TEST);
  GL::depthFunc(GL::LEQUAL);

  GL::activeTextureARB(GL::TEXTURE1_ARB);
  GL::texEnvi(GL::TEXTURE_ENV, GL::TEXTURE_ENV_MODE, GL::MODULATE);
  GL::disable(GL::TEXTURE_2D);

  GL::activeTextureARB(GL::TEXTURE0_ARB);
  GL::texEnvi(GL::TEXTURE_ENV, GL::TEXTURE_ENV_MODE, GL::MODULATE);
  GL::enable(GL::TEXTURE_2D);

  GL::enableClientState(GL::VERTEX_ARRAY);

  GL::enable(GL::CULL_FACE);
  GL::cullFace(GL::FRONT);
  GL::enable(GL::BLEND);
  GL::blendFunc(GL::ONE, GL::ZERO);

  GL::disable(GL::ALPHA_TEST);
  GL::polygonOffset(0, 0);
  GL::depthMask(GL::TRUE);
  GL::bindTexture(GL::TEXTURE_2D, 0);
  GL::disableClientState(GL::COLOR_ARRAY);
  GL::enableClientState(GL::TEXTURE_COORD_ARRAY);

  st_coloring = true;
  st_cullFace = Front;
  st_sourceBlend = GL::ONE;
  st_destBlend = GL::ZERO;
  st_alphaFunc = All;
  st_polygonOffset = false;
  st_depthWrite = true;
  st_texture = 0;
  st_texturing = true;
  st_vertexColors = false;
  st_withAlpha = false;
  st_texCoords = User;
  st_tcMod = false;
}

void Shader::setVertices(const void* vertices, uint stride)
{
  GL::vertexPointer(3, GL::FLOAT, stride, vertices);

  vertexStride = stride;
  vertexPointer = reinterpret_cast<const char*>(vertices);
}

void Shader::setColors(const void* colors, uint stride)
{
  GL::colorPointer(3, GL::UNSIGNED_BYTE, stride, colors);

  colorStride = stride;
  colorPointer = reinterpret_cast<const char*>(colors);
  st_withAlpha = false;
}

void Shader::setTextureCoords(const void* coords, uint stride)
{
  if(st_texCoords == Texture)
    GL::texCoordPointer(2, GL::FLOAT, stride, coords);

  texCoordStride = stride;
  texCoordPointer = reinterpret_cast<const char*>(coords);
}

void Shader::setLightmapCoords(const void* coords, uint stride)
{
  if(st_texCoords == Lightmap)
    GL::texCoordPointer(2, GL::FLOAT, stride, coords);

  lightCoordStride = stride;
  lightCoordPointer = reinterpret_cast<const char*>(coords);
}

void Shader::setNormals(const void* normals, uint stride)
{
  GL::normalPointer(GL::FLOAT, stride, normals);

  normalStride = stride;
  normalPointer = reinterpret_cast<const char*>(normals);
}

void Shader::set2DMode()
{
  if(mode2D)
    return;

  GL::depthMask(GL::FALSE);
  GL::depthFunc(GL::ALWAYS);

  GL::viewport(0, 0, GL::config.width, GL::config.height);
  GL::scissor(0, 0, GL::config.width, GL::config.height);

  GL::matrixMode(GL::PROJECTION);
  GL::pushMatrix();
  GL::loadIdentity();
  GL::ortho(0, GL::config.width, GL::config.height, 0, 0, 1);

  GL::matrixMode(GL::MODELVIEW);
  GL::pushMatrix();
  GL::loadIdentity();

  mode2D = true;
}

void Shader::set3DMode()
{
  if(!mode2D)
    return;

  GL::matrixMode(GL::MODELVIEW);
  GL::popMatrix();
  GL::matrixMode(GL::PROJECTION);
  GL::popMatrix();

  GL::depthMask(GL::TRUE);
  GL::depthFunc(GL::LEQUAL);

  mode2D = false;
}

Shader::~Shader()
{
}

// vim: ts=2 sw=2 et
