/***************************************************************************
                             skin.cc  -  Skin routines
                               -------------------
      copyright            : (C) 2003 by Morten Hustveit
      email                : morten@debian.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <map>

#include <espace/skin.h>
#include <espace/file.h>
#include <espace/image.h>
#include <espace/opengl.h>
#include <espace/output.h>
#include <espace/shader.h>

class Skin::Internal
{
public:

  std::map<String, Texture> textures;
};

class Skin::StaticInternal
{
public:

  StaticInternal()
    : nextHandle(1)
  {
  }

  std::map<uint, Skin*> handles;
  uint                  nextHandle;
};

Skin::StaticInternal* Skin::s = new Skin::StaticInternal;

Skin::Skin()
{
  m = new Internal;
}

Skin::~Skin()
{
  delete m;
}

uint Skin::acquire(const char* _name)
{
  String name = String(_name).replace('\\', '/');

  for(std::map<uint, Skin*>::iterator i = s->handles.begin(); i != s->handles.end(); ++i)
    if(i->second->name == name)
    {
      ++i->second->refCount;

      return i->first;
    }

  File file(name);

  if(!file.isOpen())
    return 0;

  Skin* skin = new Skin;

  skin->name = name;
  skin->refCount = 1;

  while(!file.eof())
  {
    String line = file.readLine();

    int comment = line.find("//");

    if(comment >= 0)
      line = line.left(comment);

    int comma = line.find(",");

    if(comma == -1)
      continue;

    String type = line.left(comma).stripWhiteSpace();
    String name = line.right(line.length() - comma - 1).stripWhiteSpace();

    if(type.isEmpty())
      continue;

    if(name[0] == '"' && name[name.length() - 1] == '"')
      name = name.mid(1, name.length() - 2);

    skin->m->textures[type].name = name;

    if(name.isEmpty())
    {
      skin->m->textures[type].handle = 0;
    }
    else if(!name.beginsWith("md3_"))
    {
      skin->m->textures[type].handle = Image::acquire(name);
    }
  }

  s->handles[s->nextHandle++] = skin;

  return s->nextHandle - 1;
}

void Skin::unacquire(uint handle)
{
  std::map<uint, Skin*>::iterator i = s->handles.find(handle);

  if(i == s->handles.end())
    return;

  if(--i->second->refCount)
    return;

  std::map<String, Texture>::iterator j;

  for(j = i->second->m->textures.begin(); j != i->second->m->textures.end(); ++j)
  {
    if(j->second.handle)
      Image::unacquire(j->second.handle);
  }

  delete i->second;

  s->handles.erase(i);
}

Skin* Skin::skinForHandle(uint handle)
{
  std::map<uint, Skin*>::iterator i = s->handles.find(handle);

  if(i == s->handles.end())
    return 0;

  return i->second;
}

void Skin::pushState(const char* surfaceName)
{
  std::map<String, Texture>::iterator i = m->textures.find(surfaceName);

  if(i == m->textures.end())
  {
    esDebug << "Can't find texture for surface \"" << surfaceName << "\"."
            << std::endl;

    return;
  }

  if(Shader::st_cullFace != Shader::Back)
  {
    Shader::st_cullFace = Shader::Back;
    GL::enable(GL::CULL_FACE);
    GL::cullFace(GL::BACK);
  }

  if(Shader::st_polygonOffset)
  {
    Shader::st_polygonOffset = false;
    GL::polygonOffset(0, 0);
  }

  if(Shader::st_texture != i->second.handle)
  {
    Shader::st_texture = i->second.handle;
    GL::bindTexture(GL::TEXTURE_2D, i->second.handle);
  }

  if(!Shader::st_texturing)
  {
    Shader::st_texturing = true;
    GL::enableClientState(GL::TEXTURE_COORD_ARRAY);
  }

  if(!Shader::st_vertexColors)
  {
    Shader::st_vertexColors = true;
    GL::enableClientState(GL::COLOR_ARRAY);
  }

  if(!Shader::st_depthWrite)
  {
    Shader::st_depthWrite = true;
    GL::depthMask(GL::TRUE);
  }
}

const Skin::Texture* Skin::texture(const String& name) const
{
  std::map<String, Texture>::const_iterator i = m->textures.find(name);

  if(i == m->textures.end())
    return 0;

  return &i->second;
}

// vim: ts=2 sw=2 et
