/***************************************************************************
                     system_win32.cc  -  Win32 platform specific routines
                               -------------------
      copyright            : (C) 2003 by Morten Hustveit
      email                : morten@debian.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <espace/cvar.h>
#include <espace/opengl.h>
#include <espace/output.h>
#include <espace/system.h>

#include <windows.h>
#include <iostream>
#include <imagehlp.h>

namespace
{
  HDC deviceContext;

  void uninitialize();
  long PASCAL eventHandler(HWND window, UINT message, UINT wParam, LONG lParam);
  const char* lastError();
  int backtrace(void **array, int size);
  LONG CALLBACK exceptionHandler(EXCEPTION_POINTERS* exInfo);
}

void System::initialize()
{
  SetUnhandledExceptionFilter(exceptionHandler);

  WNDCLASS windowClass;

  memset(&windowClass, 0, sizeof(windowClass));
  windowClass.lpfnWndProc = eventHandler;
  windowClass.hInstance = GetModuleHandle(0);
  windowClass.hCursor = LoadCursor(0, IDC_ARROW);
  windowClass.hbrBackground = (HBRUSH) GetStockObject(BLACK_BRUSH);
  windowClass.lpszClassName = "RenderWindowClass";
  windowClass.style = CS_HREDRAW | CS_VREDRAW | CS_OWNDC;

  if(!RegisterClass(&windowClass))
  {
    esError << "Failed to register window class." << std::endl;

    exit(EXIT_FAILURE);
  }

  CVar videoMode = CVar::acquire("r_mode", "3", CVar::Archive);

  uint width, height;
  switch(videoMode.integer)
  {
  case 0: width = 320; height = 240; break;
  case 1: width = 400; height = 300; break;
  case 2: width = 512; height = 384; break;
  case 3: width = 640; height = 480; break;
  case 4: width = 800; height = 600; break;
  case 5: width = 960; height = 720; break;
  case 6: width = 1024; height = 768; break;
  case 7: width = 1152; height = 864; break;
  case 8: width = 1280; height = 960; break;
  case 9: width = 1600; height = 1200; break;
  case 10: width = 2048; height = 1536; break;
  case 11: width = 856; height = 480; break;
  default: width = 640; height = 480;
  }

  GL::config.width = width;
  GL::config.height = height;

  HWND window = CreateWindow("RenderWindowClass", "RenderWindow",
                             WS_POPUP | WS_CLIPSIBLINGS | WS_CLIPCHILDREN, 0, 0,
                             width, height, 0, 0, GetModuleHandle(0), 0);

  if(!window)
  {
    esError << "Failed to create window." << std::endl;

    exit(EXIT_FAILURE);
  }

  ShowWindow(window, SW_SHOWDEFAULT);
  UpdateWindow(window);

  deviceContext = GetDC(window);

  if(!deviceContext)
  {
    esError << "Failed to create device context." << std::endl;

    exit(EXIT_FAILURE);
  }

  CVar fullscreen = CVar::acquire("r_fullscreen", "1", CVar::Archive);

  if(fullscreen.integer)
  {
    DEVMODE deviceMode;

    memset(&deviceMode, 0, sizeof(deviceMode));
    deviceMode.dmSize = sizeof(deviceMode);
    deviceMode.dmFields = DM_PELSWIDTH | DM_PELSHEIGHT | DM_BITSPERPEL;
    deviceMode.dmPelsWidth = GL::config.width;
    deviceMode.dmPelsHeight = GL::config.height;
    deviceMode.dmBitsPerPel = 24;

    ChangeDisplaySettings(&deviceMode, CDS_FULLSCREEN);

    // XXX: Keep current device mode, and reset in "uninitialize".
  }

  PIXELFORMATDESCRIPTOR pixelFormat;

  memset(&pixelFormat, 0, sizeof(pixelFormat));
  pixelFormat.nSize = sizeof(pixelFormat);
  pixelFormat.nVersion = 1;
  pixelFormat.dwFlags = PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL
                      | PFD_DOUBLEBUFFER;
  pixelFormat.iPixelType = PFD_TYPE_RGBA;
  pixelFormat.cColorBits = 24;
  pixelFormat.cDepthBits = 24;
  pixelFormat.cAlphaBits = 0;
  pixelFormat.cStencilBits = 8;
  pixelFormat.iLayerType = PFD_MAIN_PLANE;

  int pixelFormatIndex = ChoosePixelFormat(deviceContext, &pixelFormat);

  if(!pixelFormatIndex)
  {
    esError << "Failed to choose pixel format: " << lastError() << std::endl;

    exit(EXIT_FAILURE);
  }

  if(!SetPixelFormat(deviceContext, pixelFormatIndex, &pixelFormat))
  {
    esError << "Failed to set pixel format: " << lastError() << std::endl;

    exit(EXIT_FAILURE);
  }

  HGLRC renderContext = wglCreateContext(deviceContext);

  if(!renderContext)
  {
    esError << "Failed to create OpenGL render context: " << lastError()
            << std::endl;

    exit(EXIT_FAILURE);
  }

  if(!wglMakeCurrent(deviceContext, renderContext))
  {
    esError << "Failed to make OpenGL render context current: " << lastError()
            << std::endl;

    exit(EXIT_FAILURE);
  }

  atexit(uninitialize);
}

void System::updateScreen()
{
  SwapBuffers(deviceContext);
}

void System::idle()
{
  SwapBuffers(deviceContext);

  MSG message;

  while(PeekMessage(&message, 0, 0, 0, PM_REMOVE))
  {
    if(message.message == WM_QUIT)
      exit(EXIT_SUCCESS);

    TranslateMessage(&message);
    DispatchMessage(&message);
  }
}

double System::time()
{
  static DWORD start;

  if(!start)
    start = GetTickCount();

  return (GetTickCount() - start) / 1000.0;
}

uint System::memoryRemaining()
{
  MEMORYSTATUS memoryStatus;

  GlobalMemoryStatus(&memoryStatus);

  return memoryStatus.dwAvailPhys;
}

void* System::dlopen(const char* _fileName)
{
  char* fileName = strdup(_fileName);
  char* c;

  while(c = strchr(fileName, '/'))
    *c = '\\';

  void* ret = reinterpret_cast<void*>(LoadLibrary(fileName));

  free(fileName);

  return ret;
}

void* System::dlsym(void* handle, char* symbol)
{
  return (void*) GetProcAddress(reinterpret_cast<HMODULE>(handle), symbol);
}

const char* System::dlerror()
{
  return "Unknown error";
}

void System::dlclose(void* handle)
{
  FreeLibrary(reinterpret_cast<HMODULE>(handle));
}

namespace
{
  void uninitialize()
  {
  }

  LONG CALLBACK exceptionHandler(EXCEPTION_POINTERS* exInfo)
  {
    std::cerr << "Fatal exception.  Stack backtrace follows." << std::endl;

    PIMAGEHLP_SYMBOL symbol;

    symbol = reinterpret_cast<PIMAGEHLP_SYMBOL>(GlobalAlloc(GMEM_FIXED, 16384));
    symbol->SizeOfStruct = 16384;
    symbol->MaxNameLength = 16384 - sizeof(PIMAGEHLP_SYMBOL);

    STACKFRAME frame;

    memset(&frame, 0, sizeof(frame));
    frame.AddrPC.Offset = exInfo->ContextRecord->Eip;
    frame.AddrStack.Offset = exInfo->ContextRecord->Esp;
    frame.AddrFrame.Offset = exInfo->ContextRecord->Ebp;
    frame.AddrPC.Mode = AddrModeFlat;
    frame.AddrStack.Mode = AddrModeFlat;
    frame.AddrFrame.Mode = AddrModeFlat;

    SymInitialize(GetCurrentProcess(), 0, TRUE);

    int i = 0;

    DWORD last = 0;

    while(StackWalk(IMAGE_FILE_MACHINE_I386, GetCurrentProcess(),
                    GetCurrentThread(), &frame, 0, 0,
                    SymFunctionTableAccess, SymGetModuleBase, 0))
    {
      if(!frame.AddrFrame.Offset)
        break;

      if(frame.AddrPC.Offset == last && i > 16)
      {
        std::cerr << "Corrupt stack." << std::endl;

        break;
      }

      last = frame.AddrPC.Offset;

      std::cerr << " " << i << ". "
                << reinterpret_cast<void*>(frame.AddrPC.Offset);

      DWORD disp;

      if(SymGetSymFromAddr(GetCurrentProcess(), frame.AddrPC.Offset, &disp,
                           symbol))
      {
        std::cerr << " " << symbol->Name << " + " << disp;
      }

      std::cerr << std::endl;

      ++i;
    }

    SymCleanup(GetCurrentProcess());

    GlobalFree(symbol);

    exit(EXIT_FAILURE);
  }

  long PASCAL eventHandler(HWND window, UINT message, UINT wParam, LONG lParam)
  {
    switch(message)
    {
    case WM_KEYDOWN:

      // XXX

      break;

    case WM_CREATE:

      break;

    case WM_CLOSE:

      DestroyWindow(window);

      break;

    case WM_DESTROY:

      PostQuitMessage(0);

      break;

    default:

      return DefWindowProc(window, message, wParam, lParam);
    }

    return 0;
  }

  char errorBuffer[256];

  const char* lastError()
  {
    DWORD messageId = GetLastError();

    FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, 0, messageId, 0, errorBuffer, 256, 0);

    return errorBuffer;
  }
}

// vim: ts=2 sw=2 et
