/***************************************************************************
                       system_x11.cc  -  X11 platform specific routines
                               -------------------
      copyright            : (C) 2003 by Morten Hustveit
      email                : morten@debian.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <deque>

#include <X11/Xlib.h>
#include <X11/extensions/xf86vmode.h>
#include <X11/keysym.h>
#include <GL/glx.h>

#include <espace/cvar.h>
#include <espace/input.h>
#include <espace/opengl.h>
#include <espace/output.h>
#include <espace/system.h>

namespace
{
  Display*            display;
  Window              window;
  XVisualInfo*        visual;
  bool                restore = false;
  XF86VidModeModeInfo oldMode;
  XF86VidModeGamma    oldGamma;

  Bool waitForMapNotify(Display*, XEvent* event, char* arg);
  int  keyNum(KeySym keySym);
  void uninitialize();
}

#ifdef LINUX
extern void initializeLinux();
extern void idleLinux();
#endif
#ifdef GNU
extern void initializeGNU();
#endif

void System::initialize()
{
#ifdef LINUX
  initializeLinux();
#endif
#ifdef GNU
  initializeGNU();
#endif

  display = XOpenDisplay(0);

  if(!display)
  {
    const char* display = getenv("DISPLAY");

    esError << "Failed to open display " << (display ? display : "(null)") << std::endl;

    exit(EXIT_FAILURE);
  }

  if(!glXQueryExtension(display, 0, 0))
  {
    esError << "No GLX extension present." << std::endl;

    exit(EXIT_FAILURE);
  }

  int attributes[] = { GLX_RGBA,
                       GLX_DOUBLEBUFFER,
                       GLX_DEPTH_SIZE, 1,
                       None };

  visual = glXChooseVisual(display, DefaultScreen(display), attributes);

  // XXX: test visual == 0

  CVar videoMode = CVar::acquire("r_mode", "3", CVar::Archive);

  uint width, height;
  switch(videoMode.integer)
  {
  case 0: width = 320; height = 240; break;
  case 1: width = 400; height = 300; break;
  case 2: width = 512; height = 384; break;
  case 3: width = 640; height = 480; break;
  case 4: width = 800; height = 600; break;
  case 5: width = 960; height = 720; break;
  case 6: width = 1024; height = 768; break;
  case 7: width = 1152; height = 864; break;
  case 8: width = 1280; height = 960; break;
  case 9: width = 1600; height = 1200; break;
  case 10: width = 2048; height = 1536; break;
  case 11: width = 856; height = 480; break;
  default: width = 640; height = 480;
  }

  GL::config.width = width;
  GL::config.height = height;

  CVar fullscreen = CVar::acquire("r_fullscreen", "1", CVar::Archive);

  if(fullscreen.integer)
  {
    XF86VidModeModeInfo** modeInfo;
    int modeCount;

    XF86VidModeGetAllModeLines(display, visual->screen, &modeCount,
                               &modeInfo);

    oldMode = *modeInfo[0];
    restore = true;

    int smallestArea = INT_MAX;
    int smallestMode = 0;

    for(int i = 0; i < modeCount; ++i)
    {
      if(modeInfo[i]->hdisplay >= width
      && modeInfo[i]->vdisplay >= height)
      {
        if(modeInfo[i]->hdisplay * modeInfo[i]->vdisplay < smallestArea)
        {
          smallestArea = modeInfo[i]->hdisplay * modeInfo[i]->vdisplay;
          smallestMode = i;
        }
      }
    }

    XF86VidModeSwitchToMode(display, visual->screen, modeInfo[smallestMode]);
    XF86VidModeSetViewPort(display, visual->screen, 0, 0);

    XFree(modeInfo);
  }

  GLXContext glxContext = glXCreateContext(display, visual, 0, GL::TRUE);

  // XXX: check glxContext == 0

  Colormap colorMap = XCreateColormap(display,
                                      RootWindow(display, visual->screen),
                                      visual->visual, AllocNone);

  Pixmap mask = XCreatePixmap(display, XRootWindow(display, 0), 1, 1, 1);

  XGCValues xgc;

  xgc.function = GXclear;

  GC gc = XCreateGC(display, mask, GCFunction, &xgc);

  XFillRectangle(display, mask, gc, 0, 0, 1, 1);

  XColor color;

  color.pixel = 0;
  color.red = 0;
  color.flags = 4;

  Cursor cursor = XCreatePixmapCursor(display, mask, mask, &color, &color,
                                      0, 0);

  XFreePixmap(display, mask);

  XFreeGC(display, gc);

  XSetWindowAttributes attr;

  attr.colormap = colorMap;
  attr.border_pixel = 0;
  attr.event_mask = StructureNotifyMask | KeyPressMask | KeyReleaseMask
                  | ButtonPressMask | ButtonReleaseMask | PointerMotionMask;
  attr.cursor = cursor;

  if(fullscreen.integer)
  {
    attr.override_redirect = True;

    window = XCreateWindow(display, RootWindow(display, visual->screen),
      0, 0, width, height,
      0, visual->depth, InputOutput, visual->visual,
      CWOverrideRedirect | CWCursor | CWColormap | CWEventMask, &attr);
    XMapRaised(display, window);
    XWarpPointer(display, None, window, 0, 0, 0, 0, 0, 0);
    XGrabPointer(display, window, True, ButtonPressMask | ButtonReleaseMask | PointerMotionMask, GrabModeAsync, GrabModeAsync, window, None, CurrentTime);
    XGrabKeyboard(display, window, True, GrabModeAsync, GrabModeAsync,
                  CurrentTime);
    // XXX: Check return value of all of the above.
  }
  else // !fullscreen.integer
  {
    window = XCreateWindow(display, RootWindow(display, visual->screen),
      0, 0, width, height,
      0, visual->depth, InputOutput, visual->visual,
      CWBorderPixel | CWCursor | CWColormap | CWEventMask, &attr);

    XMapWindow(display, window);
  }

  XF86VidModeGamma gamma;
  gamma.red = 1.3;
  gamma.green = 1.3;
  gamma.blue = 1.3;

  XF86VidModeGetGamma(display, visual->screen, &oldGamma);
  XF86VidModeSetGamma(display, visual->screen, &gamma);

  XEvent event;

  XIfEvent(display, &event, waitForMapNotify, reinterpret_cast<char*>(window));

  glXMakeCurrent(display, window, glxContext);

  atexit(uninitialize);
}

void System::updateScreen()
{
  glXSwapBuffers(display, window);
}

void System::idle()
{
#ifdef LINUX
  idleLinux();
#endif

  glXSwapBuffers(display, window);

  typedef std::pair<int, char> KeyEvent;

  std::deque<KeyEvent> keyEvents;

  while(XPending(display))
  {
    XEvent event;

    XNextEvent(display, &event);

    switch(event.type)
    {
    case KeyPress:

      {
        char letter;

        KeySym keySym = XLookupKeysym(&event.xkey, 0);
        XLookupString(&event.xkey, &letter, 1, 0, 0);

        int key = keyNum(keySym);

        if(!key)
          break;

        keyEvents.push_back(KeyEvent(key, letter));
      }

      break;

    case KeyRelease:

      {
        KeySym keySym = XLookupKeysym(&event.xkey, 0);

        int key = keyNum(keySym);

        if(!key)
          break;

        keyEvents.push_back(KeyEvent(-key, 0));
      }

      break;

    case MotionNotify:

      {
        int centerX = GL::config.width / 2;
        int centerY = GL::config.height / 2;

        if(event.xmotion.x == centerX && event.xmotion.y == centerY)
          break;

        int deltaX = event.xmotion.x - centerX;
        int deltaY = event.xmotion.y - centerY;

        Input::mouseMoved(deltaX, deltaY);

        XWarpPointer(display, None, window, 0, 0, 0, 0, centerX, centerY);
      }

      break;

    case ButtonPress:

      {
        int button = event.xbutton.button - 1;

        button = (button == 1) ? 2
               : (button == 2) ? 1
               : button;

        Input::keyPressed(Input::K_MOUSE1 + button, 0);
      }

      break;

    case ButtonRelease:

      {
        int button = event.xbutton.button - 1;

        button = (button == 1) ? 2
               : (button == 2) ? 1
               : button;

        Input::keyReleased(Input::K_MOUSE1 + button);
      }

      break;

    case ConfigureNotify:

      GL::config.width = event.xconfigure.width;
      GL::config.height = event.xconfigure.height;

      break;
    }
  }

  // This is the only reasonable way to skip repeat events; the alternative
  // is to disable them GLOBALLY, which will also result in keyboard repeat
  // staying disabled if the application crashes.
  for(std::deque<KeyEvent>::iterator i = keyEvents.begin();
      i != keyEvents.end(); ++i)
  {
    if(i->first < 0)
    {
      if((i + 1) == keyEvents.end())
      {
        Input::keyReleased(-i->first);
      }
      else
      {
        if((i + 1)->first != -i->first)
        {
          Input::keyReleased(-i->first);
        }
        else
        {
          ++i;
        }
      }
    }
    else
    {
      Input::keyPressed(i->first, i->second);
    }
  }
}

namespace
{
  void uninitialize()
  {
    if(restore)
      XF86VidModeSwitchToMode(display, visual->screen, &oldMode);

    XF86VidModeSetGamma(display, visual->screen, &oldGamma);

    XFlush(display);

    exit(EXIT_SUCCESS);
  }

  Bool waitForMapNotify(Display*, XEvent* event, char* arg)
  {
    return (event->type == MapNotify) && (event->xmap.window == reinterpret_cast<Window>(arg));
  }

  int keyNum(KeySym keySym)
  {
    switch(keySym)
    {
    case XK_Tab: return Input::K_TAB;
    case XK_Escape: return Input::K_ESCAPE;
    case XK_Return: return Input::K_ENTER;
    case XK_KP_Enter: return Input::K_KP_ENTER;
    case XK_space: return Input::K_SPACE;
    case XK_BackSpace: return Input::K_BACKSPACE;
    case XK_Caps_Lock: return Input::K_CAPSLOCK;
    case XK_KP_Up: return Input::K_KP_UPARROW;
    case XK_KP_Down: return Input::K_KP_DOWNARROW;
    case XK_KP_Left: return Input::K_KP_LEFTARROW;
    case XK_KP_Right: return Input::K_KP_RIGHTARROW;
    case XK_Up: return Input::K_UPARROW;
    case XK_Down: return Input::K_DOWNARROW;
    case XK_Left: return Input::K_LEFTARROW;
    case XK_Right: return Input::K_RIGHTARROW;
    case XK_Alt_L: // XXX
    case XK_Alt_R: return Input::K_ALT;
    case XK_Control_L:
    case XK_Control_R: return Input::K_CTRL;
    case XK_Shift_L:
    case XK_Shift_R: return Input::K_SHIFT;
    case XK_KP_Insert: return Input::K_KP_INS;
    case XK_KP_Delete: return Input::K_KP_DEL;
    case XK_KP_Page_Up: return Input::K_KP_PGUP;
    case XK_KP_Page_Down: return Input::K_KP_PGDN;
    case XK_KP_Home: return Input::K_KP_HOME;
    case XK_KP_End: return Input::K_KP_END;
    case XK_Insert: return Input::K_INS;
    case XK_Delete: return Input::K_DEL;
    case XK_Page_Up: return Input::K_PGUP;
    case XK_Page_Down: return Input::K_PGDN;
    case XK_Home: return Input::K_HOME;
    case XK_End: return Input::K_END;
    case XK_F1: return Input::K_F1;
    case XK_F2: return Input::K_F2;
    case XK_F3: return Input::K_F3;
    case XK_F4: return Input::K_F4;
    case XK_F5: return Input::K_F5;
    case XK_F6: return Input::K_F6;
    case XK_F7: return Input::K_F7;
    case XK_F8: return Input::K_F8;
    case XK_F9: return Input::K_F9;
    case XK_F10: return Input::K_F10;
    case XK_F11: return Input::K_F11;
    case XK_F12: return Input::K_F12;
    case XK_F13: return Input::K_F13;
    case XK_F14: return Input::K_F14;
    case XK_F15: return Input::K_F15;
    case XK_Pause: return Input::K_PAUSE;
    case XK_KP_Multiply: return Input::K_KP_STAR;
    case XK_Pointer_Button1: return Input::K_MOUSE1;
    case XK_Pointer_Button2: return Input::K_MOUSE2;
    case XK_Pointer_Button3: return Input::K_MOUSE3;
    case XK_Pointer_Button4: return Input::K_MOUSE4;
    case XK_Pointer_Button5: return Input::K_MOUSE5;
    case XK_Pointer_Up: return Input::K_MWHEELUP;
    case XK_Pointer_Down: return Input::K_MWHEELDOWN;
    default: return toupper(keySym);
    }
  }
}

// vim: ts=2 sw=2 et
